@@ -29,6 +29,8 @@
 #include "igt.h"
 #include "igt_x86.h"
 #include "intel_bufops.h"
+#include "intel_mocs.h"
+#include "intel_pat.h"
 #include "xe/xe_ioctl.h"
 #include "xe/xe_query.h"
 
@@ -91,6 +93,7 @@
 #undef TILE_Yf
 #undef TILE_Ys
 #undef TILE_4
+#undef TILE_64
 
 #define TILE_DEF(x) (1 << (x))
 #define TILE_NONE   TILE_DEF(I915_TILING_NONE)
@@ -99,6 +102,7 @@
 #define TILE_Yf     TILE_DEF(I915_TILING_Yf)
 #define TILE_Ys     TILE_DEF(I915_TILING_Ys)
 #define TILE_4      TILE_DEF(I915_TILING_4)
+#define TILE_64     TILE_DEF(I915_TILING_64)
 
 #define CCS_OFFSET(buf) (buf->ccs[0].offset)
 #define CCS_SIZE(gen, buf) \
@@ -112,11 +116,11 @@ struct buf_ops {
 	int gen_start;
 	int gen_end;
 	unsigned int intel_gen;
+	uint32_t devid;
 	uint32_t supported_tiles;
 	uint32_t supported_hw_tiles;
 	uint32_t swizzle_x;
 	uint32_t swizzle_y;
-	uint32_t swizzle_tile4;
 	bo_copy linear_to;
 	bo_copy linear_to_x;
 	bo_copy linear_to_y;
@@ -140,6 +144,7 @@ static const char *tiling_str(uint32_t tiling)
 	case I915_TILING_Yf:   return "Yf";
 	case I915_TILING_Ys:   return "Ys";
 	case I915_TILING_4:    return "4";
+	case I915_TILING_64:   return "64";
 	default:               return "UNKNOWN";
 	}
 }
@@ -289,6 +294,17 @@ static unsigned long swizzle_addr(void *ptr, uint32_t swizzle)
 	}
 }
 
+static void *linear_ptr(void *ptr,
+			unsigned int x, unsigned int y,
+			unsigned int stride, unsigned int cpp)
+{
+	int pos;
+
+	pos = (stride/cpp * y + x) * cpp;
+
+	return ptr + pos;
+}
+
 static void *x_ptr(void *ptr,
 		   unsigned int x, unsigned int y,
 		   unsigned int stride, unsigned int cpp)
@@ -374,7 +390,6 @@ static void *tile4_ptr(void *ptr,
 		((tile_y & 3) << __builtin_ctz(owords)) +
 		(tile_x & (owords - 1));
 	igt_assert((pos & (cpp - 1)) == 0);
-	pos = pos >> __builtin_ctz(cpp);
 
 	return ptr + pos;
 }
@@ -416,6 +431,9 @@ static tile_fn __get_tile_fn_ptr(int tiling)
 	tile_fn fn = NULL;
 
 	switch (tiling) {
+	case I915_TILING_NONE:
+		fn = linear_ptr;
+		break;
 	case I915_TILING_X:
 		fn = x_ptr;
 		break;
@@ -596,6 +614,13 @@ static void __copy_linear_to(int fd, struct intel_buf *buf,
 	munmap(map, buf->surface[0].size);
 }
 
+static void copy_linear_to_none(struct buf_ops *bops, struct intel_buf *buf,
+				uint32_t *linear)
+{
+	DEBUGFN();
+	__copy_linear_to(bops->fd, buf, linear, I915_TILING_NONE, 0);
+}
+
 static void copy_linear_to_x(struct buf_ops *bops, struct intel_buf *buf,
 			     uint32_t *linear)
 {
@@ -628,7 +653,7 @@ static void copy_linear_to_tile4(struct buf_ops *bops, struct intel_buf *buf,
 				 uint32_t *linear)
 {
 	DEBUGFN();
-	__copy_linear_to(bops->fd, buf, linear, I915_TILING_4, bops->swizzle_tile4);
+	__copy_linear_to(bops->fd, buf, linear, I915_TILING_4, 0);
 }
 
 static void __copy_to_linear(int fd, struct intel_buf *buf,
@@ -653,6 +678,13 @@ static void __copy_to_linear(int fd, struct intel_buf *buf,
 	munmap(map, buf->surface[0].size);
 }
 
+static void copy_none_to_linear(struct buf_ops *bops, struct intel_buf *buf,
+				uint32_t *linear)
+{
+	DEBUGFN();
+	__copy_to_linear(bops->fd, buf, linear, I915_TILING_NONE, 0);
+}
+
 static void copy_x_to_linear(struct buf_ops *bops, struct intel_buf *buf,
 			     uint32_t *linear)
 {
@@ -774,6 +806,10 @@ void intel_buf_to_linear(struct buf_ops *bops, struct intel_buf *buf,
 		igt_assert(bops->ys_to_linear);
 		bops->ys_to_linear(bops, buf, linear);
 		break;
+	case I915_TILING_4:
+		igt_assert(bops->tile4_to_linear);
+		bops->tile4_to_linear(bops, buf, linear);
+		break;
 	}
 
 	if (buf->compression)
@@ -806,25 +842,65 @@ void linear_to_intel_buf(struct buf_ops *bops, struct intel_buf *buf,
 		igt_assert(bops->linear_to_ys);
 		bops->linear_to_ys(bops, buf, linear);
 		break;
+	case I915_TILING_4:
+		igt_assert(bops->linear_to_tile4);
+		bops->linear_to_tile4(bops, buf, linear);
+		break;
 	}
 
 	if (buf->compression)
 		__copy_ccs(bops, buf, linear, CCS_LINEAR_TO_BUF);
 }
 
+static uint32_t __get_min_stride(uint32_t width, uint32_t bpp, int tiling)
+{
+	switch (tiling) {
+	case I915_TILING_NONE:
+		return width * bpp / 8;
+	case I915_TILING_X:
+		return ALIGN(width * bpp / 8, 512);
+	case I915_TILING_64:
+		if (bpp == 8)
+			return ALIGN(width, 256);
+		else if (bpp == 16 || bpp == 32)
+			return ALIGN(width * bpp / 8, 512);
+		return ALIGN(width * bpp / 8, 1024);
+
+	default:
+		return ALIGN(width * bpp / 8, 128);
+	}
+}
+
+static uint32_t __get_aligned_height(uint32_t height, uint32_t bpp, int tiling)
+{
+	switch (tiling) {
+	case I915_TILING_NONE:
+		return height;
+	case I915_TILING_X:
+		return ALIGN(height, 8);
+	case I915_TILING_64:
+		if (bpp == 8)
+			return ALIGN(height, 256);
+		else if (bpp == 16 || bpp == 32)
+			return ALIGN(height, 128);
+		return ALIGN(height, 64);
+	default:
+		return ALIGN(height, 32);
+	}
+}
+
 static void __intel_buf_init(struct buf_ops *bops,
 			     uint32_t handle,
 			     struct intel_buf *buf,
 			     int width, int height, int bpp, int alignment,
 			     uint32_t req_tiling, uint32_t compression,
 			     uint64_t bo_size, int bo_stride,
-			     uint64_t region)
+			     uint64_t region, uint8_t pat_index,
+			     uint8_t mocs_index)
 {
 	uint32_t tiling = req_tiling;
 	uint64_t size;
-	uint32_t devid;
-	int tile_width;
-	int align_h = 1;
+	int tile_width, aligned_height;
 
 	igt_assert(bops);
 	igt_assert(buf);
@@ -835,11 +911,31 @@ static void __intel_buf_init(struct buf_ops *bops,
 	memset(buf, 0, sizeof(*buf));
 
 	buf->bops = bops;
+	buf->width = width;
+	buf->height = height;
+	buf->tiling = tiling;
+	buf->bpp = bpp;
+	buf->compression = compression;
 	buf->addr.offset = INTEL_BUF_INVALID_ADDRESS;
+	buf->pat_index = pat_index;
+	if (mocs_index == DEFAULT_MOCS_INDEX)
+		mocs_index = intel_get_uc_mocs_index(bops->fd);
+	buf->mocs_index = mocs_index;
 	IGT_INIT_LIST_HEAD(&buf->link);
-	buf->mocs = INTEL_BUF_MOCS_DEFAULT;
 
-	if (compression) {
+	tile_width = __get_min_stride(width, bpp, tiling);
+	aligned_height = __get_aligned_height(height, bpp, tiling);
+
+	if (bo_stride)
+		buf->surface[0].stride = bo_stride;
+	else
+		buf->surface[0].stride = tile_width;
+
+	size = buf->surface[0].size = buf->surface[0].stride * aligned_height;
+
+	if (compression && !HAS_FLATCCS(buf_ops_get_devid(bops))) {
+		int aux_width, aux_height;
+
 		igt_require(bops->intel_gen >= 9);
 		igt_assert(req_tiling == I915_TILING_Y ||
 			   req_tiling == I915_TILING_Yf ||
@@ -850,59 +946,12 @@ static void __intel_buf_init(struct buf_ops *bops,
 		 * CCS units, that is 4 * 64 bytes. These 4 CCS units are in
 		 * turn mapped by one L1 AUX page table entry.
 		 */
-		if (bo_stride)
-			buf->surface[0].stride = bo_stride;
-		else if (bops->intel_gen >= 12)
-			buf->surface[0].stride = ALIGN(width * (bpp / 8), 128 * 4);
-		else
-			buf->surface[0].stride = ALIGN(width * (bpp / 8), 128);
+		aux_width = intel_buf_ccs_width(bops->intel_gen, buf);
+		aux_height = intel_buf_ccs_height(bops->intel_gen, buf);
 
-		if (bops->intel_gen >= 12)
-			height = ALIGN(height, 32);
-
-		buf->surface[0].size = buf->surface[0].stride * height;
-		buf->tiling = tiling;
-		buf->bpp = bpp;
-		buf->compression = compression;
-
-		if (!HAS_FLATCCS(intel_get_drm_devid(bops->fd))) {
-			int aux_width, aux_height;
-
-			aux_width = intel_buf_ccs_width(bops->intel_gen, buf);
-			aux_height = intel_buf_ccs_height(bops->intel_gen, buf);
-
-			buf->ccs[0].offset = buf->surface[0].stride * ALIGN(height, 32);
-			buf->ccs[0].stride = aux_width;
-			size = buf->ccs[0].offset + aux_width * aux_height;
-		} else {
-			size = buf->ccs[0].offset;
-		}
-	} else {
-		if (tiling) {
-			devid =  intel_get_drm_devid(bops->fd);
-			tile_width = get_stride(devid, tiling);
-			if (bo_stride)
-				buf->surface[0].stride = bo_stride;
-			else
-				buf->surface[0].stride = ALIGN(width * (bpp / 8), tile_width);
-			align_h = tiling == I915_TILING_X ? 8 : 32;
-		} else {
-			if (bo_stride)
-				buf->surface[0].stride = bo_stride;
-			else
-				buf->surface[0].stride = ALIGN(width * (bpp / 8), alignment ?: 1);
-		}
-
-		buf->surface[0].size = buf->surface[0].stride * height;
-		buf->tiling = tiling;
-		buf->bpp = bpp;
-
-		size = buf->surface[0].stride * ALIGN(height, align_h);
-	}
-
-	if (bo_size > 0) {
-		igt_assert(bo_size >= size);
-		size = bo_size;
+		buf->ccs[0].offset = buf->surface[0].stride * ALIGN(height, 32);
+		buf->ccs[0].stride = aux_width;
+		size = buf->ccs[0].offset + aux_width * aux_height;
 	}
 
 	/* Store buffer size to avoid mistakes in calculating it again */
@@ -915,17 +964,28 @@ static void __intel_buf_init(struct buf_ops *bops,
 	buf->region = region;
 
 	if (!handle) {
+		if (!bo_size)
+			bo_size = size;
+
 		if (bops->driver == INTEL_DRIVER_I915) {
-			if (__gem_create_in_memory_regions(bops->fd, &buf->handle, &size, region))
-				igt_assert_eq(__gem_create(bops->fd, &size, &buf->handle), 0);
+			if (__gem_create_in_memory_regions(bops->fd, &buf->handle, &bo_size, region))
+				igt_assert_eq(__gem_create(bops->fd, &bo_size, &buf->handle), 0);
 		} else {
-			size = ALIGN(size, xe_get_default_alignment(bops->fd));
-			buf->handle = xe_bo_create_flags(bops->fd, 0, size, region);
+			uint16_t cpu_caching = __xe_default_cpu_caching(bops->fd, region, 0);
+
+			if (AT_LEAST_GEN(bops->devid, 20) && compression)
+				cpu_caching = DRM_XE_GEM_CPU_CACHING_WC;
+
+			bo_size = ALIGN(bo_size, xe_get_default_alignment(bops->fd));
+			buf->handle = xe_bo_create_caching(bops->fd, 0, bo_size, region, 0,
+							   cpu_caching);
 		}
 	}
 
+	igt_assert(bo_size >= size);
+
 	/* Store gem bo size */
-	buf->bo_size = size;
+	buf->bo_size = bo_size;
 
 	if (bops->driver == INTEL_DRIVER_I915)
 		set_hw_tiled(bops, buf);
@@ -953,11 +1013,17 @@ void intel_buf_init(struct buf_ops *bops,
 		    uint32_t tiling, uint32_t compression)
 {
 	uint64_t region;
+	uint8_t pat_index = DEFAULT_PAT_INDEX;
+
+	if (compression && AT_LEAST_GEN(bops->devid, 20))
+		pat_index = intel_get_pat_idx_uc_comp(bops->fd);
 
 	region = bops->driver == INTEL_DRIVER_I915 ? I915_SYSTEM_MEMORY :
 						     system_memory(bops->fd);
 	__intel_buf_init(bops, 0, buf, width, height, bpp, alignment,
-			 tiling, compression, 0, 0, region);
+			 tiling, compression, 0, 0, region,
+			 pat_index,
+			 DEFAULT_MOCS_INDEX);
 
 	intel_buf_set_ownership(buf, true);
 }
@@ -973,8 +1039,15 @@ void intel_buf_init_in_region(struct buf_ops *bops,
 			      uint32_t tiling, uint32_t compression,
 			      uint64_t region)
 {
+	uint8_t pat_index = DEFAULT_PAT_INDEX;
+
+	if (compression && AT_LEAST_GEN(bops->devid, 20))
+		pat_index = intel_get_pat_idx_uc_comp(bops->fd);
+
 	__intel_buf_init(bops, 0, buf, width, height, bpp, alignment,
-			 tiling, compression, 0, 0, region);
+			 tiling, compression, 0, 0, region,
+			 pat_index,
+			 DEFAULT_MOCS_INDEX);
 
 	intel_buf_set_ownership(buf, true);
 }
@@ -1008,7 +1081,7 @@ void intel_buf_close(struct buf_ops *bops, struct intel_buf *buf)
 }
 
 /**
- * intel_buf_init_using_handle
+ * intel_buf_init_using_handle_and_size
  * @bops: pointer to buf_ops
  * @handle: BO handle created by the caller
  * @buf: pointer to intel_buf structure to be filled
@@ -1018,6 +1091,7 @@ void intel_buf_close(struct buf_ops *bops, struct intel_buf *buf)
  * @alignment: alignment of the stride for linear surfaces
  * @tiling: surface tiling
  * @compression: surface compression type
+ * @size: real bo size
  *
  * Function configures BO handle within intel_buf structure passed by the caller
  * (with all its metadata - width, height, ...). Useful if BO was created
@@ -1026,14 +1100,24 @@ void intel_buf_close(struct buf_ops *bops, struct intel_buf *buf)
  * Note: intel_buf_close() can be used because intel_buf is aware it is not
  * buffer owner so it won't close it underneath.
  */
-void intel_buf_init_using_handle(struct buf_ops *bops,
-				 uint32_t handle,
-				 struct intel_buf *buf,
-				 int width, int height, int bpp, int alignment,
-				 uint32_t req_tiling, uint32_t compression)
+void intel_buf_init_using_handle_and_size(struct buf_ops *bops,
+					  uint32_t handle,
+					  struct intel_buf *buf,
+					  int width, int height, int bpp, int alignment,
+					  uint32_t req_tiling, uint32_t compression,
+					  uint64_t size)
 {
+	uint8_t pat_index = DEFAULT_PAT_INDEX;
+
+	igt_assert(handle);
+	igt_assert(size);
+
+	if (compression && AT_LEAST_GEN(bops->devid, 20))
+		pat_index = intel_get_pat_idx_uc_comp(bops->fd);
+
 	__intel_buf_init(bops, handle, buf, width, height, bpp, alignment,
-			 req_tiling, compression, 0, 0, -1);
+			 req_tiling, compression, size, 0, -1, pat_index,
+			 DEFAULT_MOCS_INDEX);
 }
 
 /**
@@ -1050,6 +1134,9 @@ void intel_buf_init_using_handle(struct buf_ops *bops,
  * @size: real bo size
  * @stride: bo stride
  * @region: region
+ * @pat_index: pat_index to use for the binding (only used on xe)
+ * @pat_index: mocs_index to use for operations using this intel_buf, like render
+ * copy.
  *
  * Function configures BO handle within intel_buf structure passed by the caller
  * (with all its metadata - width, height, ...). Useful if BO was created
@@ -1067,10 +1154,13 @@ void intel_buf_init_full(struct buf_ops *bops,
 			 uint32_t compression,
 			 uint64_t size,
 			 int stride,
-			 uint64_t region)
+			 uint64_t region,
+			 uint8_t pat_index,
+			 uint8_t mocs_index)
 {
 	__intel_buf_init(bops, handle, buf, width, height, bpp, alignment,
-			 req_tiling, compression, size, stride, region);
+			 req_tiling, compression, size, stride, region,
+			 pat_index, mocs_index);
 }
 
 /**
@@ -1105,7 +1195,7 @@ struct intel_buf *intel_buf_create(struct buf_ops *bops,
 }
 
 /**
- * intel_buf_create_using_handle
+ * intel_buf_create_using_handle_and_size
  * @bops: pointer to buf_ops
  * @handle: BO handle created by the caller
  * @width: surface width
@@ -1114,42 +1204,25 @@ struct intel_buf *intel_buf_create(struct buf_ops *bops,
  * @alignment: alignment of the stride for linear surfaces
  * @tiling: surface tiling
  * @compression: surface compression type
+ * @size: real bo size
  *
  * Function creates intel_buf with passed BO handle from the caller. Doesn't
  * take ownership of the buffer. close()/destroy() paths doesn't close
  * passed handle unless buffer will take ownership using set_ownership().
  */
-struct intel_buf *intel_buf_create_using_handle(struct buf_ops *bops,
-						uint32_t handle,
-						int width, int height,
-						int bpp, int alignment,
-						uint32_t req_tiling,
-						uint32_t compression)
-{
-	struct intel_buf *buf;
-
-	igt_assert(bops);
-
-	buf = calloc(1, sizeof(*buf));
-	igt_assert(buf);
-
-	intel_buf_init_using_handle(bops, handle, buf, width, height, bpp,
-				    alignment, req_tiling, compression);
-
-	return buf;
-}
-
 struct intel_buf *intel_buf_create_using_handle_and_size(struct buf_ops *bops,
 							 uint32_t handle,
 							 int width, int height,
 							 int bpp, int alignment,
 							 uint32_t req_tiling,
 							 uint32_t compression,
-							 uint64_t size,
-							 int stride)
+							 uint64_t size)
 {
+	igt_assert(handle);
+	igt_assert(size);
 	return intel_buf_create_full(bops, handle, width, height, bpp, alignment,
-				     req_tiling, compression, size, stride, -1);
+				     req_tiling, compression, size, 0, -1,
+				     DEFAULT_PAT_INDEX, DEFAULT_MOCS_INDEX);
 }
 
 struct intel_buf *intel_buf_create_full(struct buf_ops *bops,
@@ -1160,7 +1233,9 @@ struct intel_buf *intel_buf_create_full(struct buf_ops *bops,
 					uint32_t compression,
 					uint64_t size,
 					int stride,
-					uint64_t region)
+					uint64_t region,
+					uint8_t pat_index,
+					uint8_t mocs_index)
 {
 	struct intel_buf *buf;
 
@@ -1170,7 +1245,8 @@ struct intel_buf *intel_buf_create_full(struct buf_ops *bops,
 	igt_assert(buf);
 
 	__intel_buf_init(bops, handle, buf, width, height, bpp, alignment,
-			 req_tiling, compression, size, stride, region);
+			 req_tiling, compression, size, stride, region,
+			 pat_index, mocs_index);
 
 	return buf;
 }
@@ -1194,37 +1270,46 @@ void intel_buf_destroy(struct intel_buf *buf)
 
 void *intel_buf_cpu_map(struct intel_buf *buf, bool write)
 {
-	int i915 = buf_ops_get_fd(buf->bops);
+	int fd = buf_ops_get_fd(buf->bops);
 
 	igt_assert(buf);
 	igt_assert(buf->ptr == NULL); /* already mapped */
 
 	buf->cpu_write = write;
-	buf->ptr = gem_mmap__cpu_coherent(i915, buf->handle, 0,
-					  buf->surface[0].size,
-					  write ? PROT_WRITE : PROT_READ);
 
-	gem_set_domain(i915, buf->handle,
-		       I915_GEM_DOMAIN_CPU,
-		       write ? I915_GEM_DOMAIN_CPU : 0);
+	if (is_xe_device(fd)) {
+		buf->ptr = xe_bo_map(fd, buf->handle, buf->bo_size);
+	} else {
+		buf->ptr = gem_mmap__cpu_coherent(fd, buf->handle, 0,
+						  buf->bo_size,
+						  write ? PROT_WRITE : PROT_READ);
+
+		gem_set_domain(fd, buf->handle,
+			       I915_GEM_DOMAIN_CPU,
+			       write ? I915_GEM_DOMAIN_CPU : 0);
+	}
 
 	return buf->ptr;
 }
 
 void *intel_buf_device_map(struct intel_buf *buf, bool write)
 {
-	int i915 = buf_ops_get_fd(buf->bops);
+	int fd = buf_ops_get_fd(buf->bops);
 
 	igt_assert(buf);
 	igt_assert(buf->ptr == NULL); /* already mapped */
 
-	buf->ptr = gem_mmap__device_coherent(i915, buf->handle, 0,
-					     buf->surface[0].size,
-					     write ? PROT_WRITE : PROT_READ);
+	if (is_xe_device(fd)) {
+		buf->ptr = xe_bo_map(fd, buf->handle, buf->bo_size);
+	} else {
+		buf->ptr = gem_mmap__device_coherent(fd, buf->handle, 0,
+						     buf->bo_size,
+						     write ? PROT_WRITE : PROT_READ);
 
-	gem_set_domain(i915, buf->handle,
-		       I915_GEM_DOMAIN_WC,
-		       write ? I915_GEM_DOMAIN_WC : 0);
+		gem_set_domain(fd, buf->handle,
+			       I915_GEM_DOMAIN_WC,
+			       write ? I915_GEM_DOMAIN_WC : 0);
+	}
 
 	return buf->ptr;
 }
@@ -1234,7 +1319,7 @@ void intel_buf_unmap(struct intel_buf *buf)
 	igt_assert(buf);
 	igt_assert(buf->ptr);
 
-	munmap(buf->ptr, buf->surface[0].size);
+	munmap(buf->ptr, buf->bo_size);
 	buf->ptr = NULL;
 }
 
@@ -1330,11 +1415,122 @@ void intel_buf_write_aux_to_png(struct intel_buf *buf, const char *filename)
 
 	__intel_buf_write_to_png(buf->bops, buf, filename, true);
 }
+static void __intel_buf_raw_write_to_png(struct buf_ops *bops,
+					 struct intel_buf *buf,
+					 const char *filename)
+{
+	cairo_surface_t *surface;
+	cairo_status_t ret;
+	uint8_t *linear;
+	int format, width, height, stride;
+
+	format = CAIRO_FORMAT_RGB24;
+	width = buf->surface[0].stride / 4;
+	height = __get_aligned_height(intel_buf_height(buf),
+				      buf->bpp, buf->tiling);
+	stride = buf->surface[0].stride;
+
+	if (is_xe_device(bops->fd))
+		linear = xe_bo_map(bops->fd, buf->handle, buf->bo_size);
+	else
+		linear = gem_mmap__device_coherent(bops->fd, buf->handle,
+						   0, buf->bo_size, PROT_READ);
+	surface = cairo_image_surface_create_for_data(linear,
+						      format, width, height,
+						      stride);
+	ret = cairo_surface_write_to_png(surface, filename);
+	igt_assert(ret == CAIRO_STATUS_SUCCESS);
+	cairo_surface_destroy(surface);
+
+	munmap(linear, buf->bo_size);
+}
+
+__attribute__((format(printf, 2, 3)))
+void intel_buf_raw_write_to_png(struct intel_buf *buf, const char *namefmt, ...)
+{
+	char *filename;
+	int ret;
+	va_list ap;
+
+	va_start(ap, namefmt);
+	ret = vasprintf(&filename, namefmt, ap);
+	igt_assert(ret >= 0);
+	va_end(ap);
+
+	__intel_buf_raw_write_to_png(buf->bops, buf, filename);
+
+	free(filename);
+}
+
+static void *alloc_aligned(uint64_t size)
+{
+	void *p;
+
+	igt_assert_eq(posix_memalign(&p, 16, size), 0);
+
+	return p;
+}
+
+void intel_buf_draw_pattern(struct buf_ops *bops, struct intel_buf *buf,
+			    int x, int y, int w, int h,
+			    int cx, int cy, int cw, int ch,
+			    bool use_alternate_colors)
+{
+	cairo_surface_t *surface;
+	cairo_pattern_t *pat;
+	cairo_t *cr;
+	void *linear;
+
+	linear = alloc_aligned(buf->surface[0].size);
+
+	surface = cairo_image_surface_create_for_data(linear,
+						      CAIRO_FORMAT_RGB24,
+						      intel_buf_width(buf),
+						      intel_buf_height(buf),
+						      buf->surface[0].stride);
+
+	cr = cairo_create(surface);
+
+	cairo_rectangle(cr, cx, cy, cw, ch);
+	cairo_clip(cr);
+
+	pat = cairo_pattern_create_mesh();
+	cairo_mesh_pattern_begin_patch(pat);
+	cairo_mesh_pattern_move_to(pat, x,   y);
+	cairo_mesh_pattern_line_to(pat, x+w, y);
+	cairo_mesh_pattern_line_to(pat, x+w, y+h);
+	cairo_mesh_pattern_line_to(pat, x,   y+h);
+	if (use_alternate_colors) {
+		cairo_mesh_pattern_set_corner_color_rgb(pat, 0, 0.0, 1.0, 1.0);
+		cairo_mesh_pattern_set_corner_color_rgb(pat, 1, 1.0, 0.0, 1.0);
+		cairo_mesh_pattern_set_corner_color_rgb(pat, 2, 1.0, 1.0, 0.0);
+		cairo_mesh_pattern_set_corner_color_rgb(pat, 3, 0.0, 0.0, 0.0);
+	} else {
+		cairo_mesh_pattern_set_corner_color_rgb(pat, 0, 1.0, 0.0, 0.0);
+		cairo_mesh_pattern_set_corner_color_rgb(pat, 1, 0.0, 1.0, 0.0);
+		cairo_mesh_pattern_set_corner_color_rgb(pat, 2, 0.0, 0.0, 1.0);
+		cairo_mesh_pattern_set_corner_color_rgb(pat, 3, 1.0, 1.0, 1.0);
+	}
+	cairo_mesh_pattern_end_patch(pat);
+
+	cairo_rectangle(cr, x, y, w, h);
+	cairo_set_source(cr, pat);
+	cairo_fill(cr);
+	cairo_pattern_destroy(pat);
+
+	cairo_destroy(cr);
+
+	cairo_surface_destroy(surface);
+
+	linear_to_intel_buf(bops, buf, linear);
+
+	free(linear);
+}
 
 #define DEFAULT_BUFOPS(__gen_start, __gen_end) \
 	.gen_start          = __gen_start, \
 	.gen_end            = __gen_end, \
-	.supported_hw_tiles = TILE_X | TILE_Y | TILE_4, \
+	.supported_hw_tiles = TILE_X | TILE_Y, \
 	.linear_to          = copy_linear_to_wc, \
 	.linear_to_x        = copy_linear_to_gtt, \
 	.linear_to_y        = copy_linear_to_gtt, \
@@ -1370,12 +1566,11 @@ static bool probe_hw_tiling(struct buf_ops *bops, uint32_t tiling,
 {
 	uint64_t size = 256 * 256;
 	uint32_t handle, buf_tiling, buf_swizzle, phys_swizzle;
-	uint32_t devid, stride;
+	uint32_t stride;
 	int ret;
 	bool is_set = false;
 
-	devid =  intel_get_drm_devid(bops->fd);
-	stride = get_stride(devid, tiling);
+	stride = get_stride(bops->devid, tiling);
 	handle = gem_create(bops->fd, size);
 
 	/* Single shot, if no fences are available we fail immediately */
@@ -1390,8 +1585,6 @@ static bool probe_hw_tiling(struct buf_ops *bops, uint32_t tiling,
 			bops->swizzle_x = buf_swizzle;
 		else if (tiling == I915_TILING_Y)
 			bops->swizzle_y = buf_swizzle;
-		else if (tiling == I915_TILING_4)
-			bops->swizzle_tile4 = buf_swizzle;
 
 		*swizzling_supported = buf_swizzle == phys_swizzle;
 	}
@@ -1401,15 +1594,6 @@ end:
 	return is_set;
 }
 
-static void *alloc_aligned(uint64_t size)
-{
-	void *p;
-
-	igt_assert_eq(posix_memalign(&p, 16, size), 0);
-
-	return p;
-}
-
 /*
  * Simple idempotency test between HW -> SW and SW -> HW BO.
  */
@@ -1496,6 +1680,7 @@ static struct buf_ops *__buf_ops_create(int fd, bool check_idempotency)
 
 	bops->fd = fd;
 	bops->intel_gen = generation;
+	bops->devid = devid;
 	bops->driver = is_i915_device(fd) ? INTEL_DRIVER_I915 :
 					    is_xe_device(fd) ? INTEL_DRIVER_XE : 0;
 	igt_assert(bops->driver);
@@ -1504,17 +1689,12 @@ static struct buf_ops *__buf_ops_create(int fd, bool check_idempotency)
 		  bops->driver == INTEL_DRIVER_I915 ? "i915" : "xe");
 
 	if (bops->driver == INTEL_DRIVER_XE) {
+		bops->linear_to = copy_linear_to_none;
+		bops->to_linear = copy_none_to_linear;
 		bops->linear_to_x = copy_linear_to_x;
 		bops->x_to_linear = copy_x_to_linear;
 		bops->linear_to_y = copy_linear_to_y;
 		bops->y_to_linear = copy_y_to_linear;
-		bops->linear_to_tile4 = copy_linear_to_tile4;
-		bops->tile4_to_linear = copy_tile4_to_linear;
-
-		bops->linear_to_yf = NULL;
-		bops->yf_to_linear = NULL;
-		bops->linear_to_ys = NULL;
-		bops->ys_to_linear = NULL;
 
 		return bops;
 	}
@@ -1540,7 +1720,7 @@ static struct buf_ops *__buf_ops_create(int fd, bool check_idempotency)
 		bool supported = probe_hw_tiling(bops, I915_TILING_X,
 						 &swizzling_supported);
 
-		if (!swizzling_supported) {
+		if (!swizzling_supported && bops->intel_gen < 12) {
 			igt_debug("Swizzling for X is not supported\n");
 			bops->supported_tiles &= ~TILE_X;
 		}
@@ -1558,7 +1738,7 @@ static struct buf_ops *__buf_ops_create(int fd, bool check_idempotency)
 		bool supported = probe_hw_tiling(bops, I915_TILING_Y,
 						 &swizzling_supported);
 
-		if (!swizzling_supported) {
+		if (!swizzling_supported && bops->intel_gen < 12) {
 			igt_debug("Swizzling for Y is not supported\n");
 			bops->supported_tiles &= ~TILE_Y;
 		}
@@ -1571,36 +1751,15 @@ static struct buf_ops *__buf_ops_create(int fd, bool check_idempotency)
 		}
 	}
 
-	if (is_hw_tiling_supported(bops, I915_TILING_4)) {
-		bool swizzling_supported;
-		bool supported = probe_hw_tiling(bops, I915_TILING_4,
-						 &swizzling_supported);
-
-		if (!swizzling_supported) {
-			igt_debug("Swizzling for 4 is not supported\n");
-			bops->supported_tiles &= ~TILE_4;
-		}
-
-		igt_debug("4 fence support: %s\n", bool_str(supported));
-		if (!supported) {
-			bops->supported_hw_tiles &= ~TILE_4;
-			bops->linear_to_tile4 = copy_linear_to_tile4;
-			bops->tile4_to_linear = copy_tile4_to_linear;
-		}
-	}
-
 	/* Disable other tiling format functions if not supported */
-	if (!is_tiling_supported(bops, I915_TILING_Yf)) {
+	if (!is_tiling_supported(bops, I915_TILING_Yf))
 		igt_debug("Yf format not supported\n");
-		bops->linear_to_yf = NULL;
-		bops->yf_to_linear = NULL;
-	}
 
-	if (!is_tiling_supported(bops, I915_TILING_Ys)) {
+	if (!is_tiling_supported(bops, I915_TILING_Ys))
 		igt_debug("Ys format not supported\n");
-		bops->linear_to_ys = NULL;
-		bops->ys_to_linear = NULL;
-	}
+
+	if (!is_tiling_supported(bops, I915_TILING_4))
+		igt_debug("Tile4 format not supported\n");
 
 	if (check_idempotency) {
 		idempotency_selftest(bops, I915_TILING_X);
@@ -1665,6 +1824,19 @@ int buf_ops_get_fd(struct buf_ops *bops)
 	return bops->fd;
 }
 
+/**
+ * buf_ops_get_devid
+ * @bops: pointer to buf_ops
+ *
+ * Returns: device id
+ */
+uint32_t buf_ops_get_devid(struct buf_ops *bops)
+{
+	igt_assert(bops);
+
+	return bops->devid;
+}
+
 /**
  * buf_ops_get_driver
  * @bops: pointer to buf_ops
@@ -1707,6 +1879,10 @@ bool buf_ops_set_software_tiling(struct buf_ops *bops,
 	}
 
 	switch (tiling) {
+	case I915_TILING_NONE:
+		igt_debug("-> use SW on tiling NONE\n");
+		break;
+
 	case I915_TILING_X:
 		if (use_software_tiling) {
 			bool supported = buf_ops_has_tiling_support(bops, tiling);
@@ -1747,6 +1923,18 @@ bool buf_ops_set_software_tiling(struct buf_ops *bops,
 		}
 		break;
 
+	case I915_TILING_4:
+		igt_debug("-> use SW on tiling 4\n");
+		break;
+
+	case I915_TILING_Yf:
+		igt_debug("-> use SW on tiling Yf\n");
+		break;
+
+	case I915_TILING_Ys:
+		igt_debug("-> use SW on tiling Ys\n");
+		break;
+
 	default:
 		igt_warn("Invalid tiling: %d\n", tiling);
 		was_changed = false;
