@@ -64,101 +64,63 @@
 #include "i915_drm.h"
 /**
  * TEST: gem create
- * Description:
- *   Ensure that basic gem_create and gem_create_ext works and that invalid input combinations
- *   are rejected.
- * Run type: FULL
+ * Description: Ensure that basic gem_create and gem_create_ext works and that
+ *		invalid input combinations are rejected.
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: uapi
+ * Functionality: gem_create IOCTL
+ * Feature: mapping
+ * Test category: GEM_Legacy
  *
  * SUBTEST: busy-create
- * Category: Infrastructure
  * Description: Create buffer objects while GPU is busy.
- * Feature: mapping
- * Functionality: buffer management
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: create-clear
- * Category: Infrastructure
  * Description: Verify that all new objects are clear.
- * Feature: mapping
- * Functionality: buffer management
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: create-ext-cpu-access-big
- * Description:
- *   Verify the extreme cases with very large objects and.
- *   I915_GEM_CREATE_EXT_FLAG_NEEDS_CPU_ACCESS
- * Feature: mapping
+ * Description: Verify the extreme cases with very large objects and
+ *		I915_GEM_CREATE_EXT_FLAG_NEEDS_CPU_ACCESS
  *
  * SUBTEST: create-ext-cpu-access-sanity-check
- * Description:
- *   Verify the basic functionally and expected ABI contract around.
- *   I915_GEM_CREATE_EXT_FLAG_NEEDS_CPU_ACCESS
- * Feature: mapping
+ * Description: Verify the basic functionally and expected ABI contract around
+ *		I915_GEM_CREATE_EXT_FLAG_NEEDS_CPU_ACCESS
  *
  * SUBTEST: create-ext-placement-all
- * Category: Infrastructure
+ * Category: Core
  * Description: Create objects in every memory region using create_ext.
- * Feature: mapping
- * Functionality: buffer management
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: create-ext-placement-each
- * Category: Infrastructure
- * Description: Create one object with memory pieces in each memory region using create_ext.
- * Feature: mapping
- * Functionality: buffer management
- * Sub-category: i915
- * Test category: GEM_Legacy
+ * Description: Create one object with memory pieces in each memory region
+ *		using create_ext.
  *
  * SUBTEST: create-ext-placement-sanity-check
- * Category: Infrastructure
  * Description: Exercise create_ext placements extension.
- * Feature: mapping
- * Functionality: buffer management
- * Sub-category: i915
- * Test category: GEM_Legacy
+ *
+ * SUBTEST: create-ext-set-pat
+ * Description:  Create GEM object with specific PAT index
  *
  * SUBTEST: create-invalid-size
- * Category: Infrastructure
- * Description: Try to create a gem object of invalid size 0 and check if ioctl returns error.
- * Feature: mapping
- * Functionality: buffer management
- * Sub-category: i915
- * Test category: GEM_Legacy
+ * Description: Try to create a gem object of invalid size 0 and check if
+ *		ioctl returns error.
  *
  * SUBTEST: create-massive
- * Category: Infrastructure
- * Description: Exercise creation of buffer object with impossible size and check for the expected error.
- * Feature: mapping
- * Functionality: buffer management
- * Sub-category: i915
- * Test category: GEM_Legacy
+ * Description: Exercise creation of buffer object with impossible size and
+ *		check for the expected error.
  *
  * SUBTEST: create-size-update
- * Category: Infrastructure
- * Description: Try to create a gem object with size 15 and check actual created size.
- * Feature: mapping
- * Functionality: buffer management
- * Sub-category: i915
- * Test category: GEM_Legacy
+ * Description: Try to create a gem object with size 15 and check actual
+ *		created size.
  *
  * SUBTEST: create-valid-nonaligned
- * Category: Infrastructure
- * Description:
- *   Try to create an object with non-aligned size, check we got one with size aligned up to page
- *   size and test we can write into the padded extra memory.
- * Feature: mapping
- * Functionality: buffer management
- * Sub-category: i915
- * Test category: GEM_Legacy
+ * Description: Try to create an object with non-aligned size, check we got one
+ *		with size aligned up to page size and test we can write into
+ *		the padded extra memory.
  *
  * SUBTEST: hog-create
  * Description: Create buffer objects while GPU is busy.
- * Feature: mapping
- * Test category: GEM_Legacy
+ *
  */
 
 IGT_TEST_DESCRIPTION("Ensure that basic gem_create and gem_create_ext works"
@@ -635,18 +597,6 @@ static void create_ext_placement_each(int fd)
 	free(regions);
 }
 
-/**
-  * TEST: GEM create uAPI
-  * Category: Infrastructure
-  * Description: Tests the DRM_IOCTL_I915_GEM_CREATE_EXT ioctl.
-  * Feature: core
-  * Functionality: Create GEM object with specific PAT index
-  * Run type: BAT
-  * Sub-category: i915
-  * Test category: GEM_Legacy
-  *
-  * SUBTEST: create-ext-set-pat
-  */
 static void create_ext_set_pat(int fd)
 {
 	struct drm_i915_gem_create_ext_set_pat setparam_pat = {
@@ -687,7 +637,7 @@ static void create_ext_set_pat(int fd)
 	 * This means that we are on a Meteor Lake and the PAT
 	 * index is already supported by the running i915
 	 */
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/*
 	 * {set|get}_caching ioctl should fail for objects created with set_pat
