@@ -50,71 +50,41 @@
 /**
  * TEST: gem ctx shared
  * Description: Test shared contexts.
- * Run type: FULL
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: CMD submission
+ * Functionality: shared contex
+ * Feature: context
+ * Test category: GEM_Legacy
  *
  * SUBTEST: Q-in-order
- * Feature: context
  *
  * SUBTEST: Q-independent
- * Feature: context
  *
  * SUBTEST: Q-out-order
- * Feature: context
  *
  * SUBTEST: Q-promotion
- * Feature: context
  *
  * SUBTEST: Q-smoketest
- * Feature: context
  *
  * SUBTEST: Q-smoketest-all
- * Feature: context
  *
  * SUBTEST: create-shared-gtt
- * Category: Desktop client
- * Feature: context
- * Functionality: context management
- * Sub-category: Media
- * Test category: GEM_Legacy
  *
  * SUBTEST: detached-shared-gtt
- * Category: Desktop client
- * Feature: context
- * Functionality: context management
- * Sub-category: Media
- * Test category: GEM_Legacy
  *
  * SUBTEST: disjoint-timelines
- * Category: Desktop client
- * Feature: context
- * Functionality: context management
- * Sub-category: Media
- * Test category: GEM_Legacy
  *
  * SUBTEST: exec-shared-gtt
- * Category: Desktop client
- * Feature: context
- * Functionality: context management
- * Sub-category: Media
- * Test category: GEM_Legacy
  *
  * SUBTEST: exec-single-timeline
- * Category: Desktop client
- * Feature: context
- * Functionality: context management
- * Test category: GEM_Legacy
  *
  * SUBTEST: exhaust-shared-gtt
- * Feature: context
  *
  * SUBTEST: exhaust-shared-gtt-lrc
- * Feature: context
  *
  * SUBTEST: single-timeline
- * Category: Desktop client
- * Feature: context
- * Functionality: context management
- * Test category: GEM_Legacy
+ *
  */
 
 #define LO 0
@@ -1047,6 +1017,12 @@ static void smoketest(int i915, const intel_ctx_cfg_t *cfg,
 		for_each_if(gem_class_can_store_dword(i915, (e)->class)) \
 			igt_dynamic_f("%s", e->name)
 
+#define for_each_nonblocking_ggtt_binder_queue(e, i915, cfg) \
+	for_each_ctx_cfg_engine(i915, cfg, e) \
+		for_each_if(gem_class_can_store_dword(i915, (e)->class) && \
+			    !gem_engine_can_block_ggtt_binder(i915, e)) \
+			igt_dynamic_f("%s", e->name)
+
 igt_main
 {
 	const struct intel_execution_engine2 *e;
@@ -1104,22 +1080,22 @@ igt_main
 			}
 
 			igt_subtest_with_dynamic("Q-independent") {
-				for_each_queue(e, i915, &cfg)
+				for_each_nonblocking_ggtt_binder_queue(e, i915, &cfg)
 					independent(i915, &cfg, e, 0);
 			}
 
 			igt_subtest_with_dynamic("Q-in-order") {
-				for_each_queue(e, i915, &cfg)
+				for_each_nonblocking_ggtt_binder_queue(e, i915, &cfg)
 					reorder(i915, &cfg, e->flags, EQUAL);
 			}
 
 			igt_subtest_with_dynamic("Q-out-order") {
-				for_each_queue(e, i915, &cfg)
+				for_each_nonblocking_ggtt_binder_queue(e, i915, &cfg)
 					reorder(i915, &cfg, e->flags, 0);
 			}
 
 			igt_subtest_with_dynamic("Q-promotion") {
-				for_each_queue(e, i915, &cfg)
+				for_each_nonblocking_ggtt_binder_queue(e, i915, &cfg)
 					promotion(i915, &cfg, e->flags);
 			}
 		}
