@@ -40,280 +40,103 @@
 /**
  * TEST: gem exec whisper
  * Feature: cmd_submission
- * Run type: FULL
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: CMD submission
+ * Functionality: execbuf
+ * Test category: GEM_Legacy
  *
  * SUBTEST: basic-contexts
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-contexts-all
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-contexts-forked
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-contexts-forked-all
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-contexts-priority
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-contexts-priority-all
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-fds
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-fds-all
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-fds-forked
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-fds-forked-all
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-fds-priority
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-fds-priority-all
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-forked
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-forked-all
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-normal
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-normal-all
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-queues
- * Category: Infrastructure
- * Functionality: command submission
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-queues-all
- * Category: Infrastructure
- * Functionality: command submission
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-queues-forked
- * Category: Infrastructure
- * Functionality: command submission
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-queues-forked-all
- * Category: Infrastructure
- * Functionality: command submission
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-queues-priority
- * Category: Infrastructure
- * Functionality: command submission
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-queues-priority-all
- * Category: Infrastructure
- * Functionality: command submission
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-sync
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: basic-sync-all
- * Category: Infrastructure
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
- *
  * SUBTEST: chain
- *
  * SUBTEST: chain-all
- *
  * SUBTEST: chain-forked
- *
  * SUBTEST: chain-forked-all
- *
  * SUBTEST: chain-interruptible
- *
  * SUBTEST: chain-interruptible-all
- *
  * SUBTEST: chain-sync
- *
  * SUBTEST: chain-sync-all
- *
  * SUBTEST: contexts-chain
- *
  * SUBTEST: contexts-chain-all
- *
  * SUBTEST: contexts-engine
- *
  * SUBTEST: contexts-forked-engine
- *
  * SUBTEST: contexts-interruptible
- *
  * SUBTEST: contexts-interruptible-all
- *
  * SUBTEST: contexts-interruptible-engine
- *
  * SUBTEST: contexts-priority-engine
- *
  * SUBTEST: contexts-sync
- *
  * SUBTEST: contexts-sync-all
- *
  * SUBTEST: contexts-sync-engine
- *
  * SUBTEST: fds-chain
- *
  * SUBTEST: fds-chain-all
- *
  * SUBTEST: fds-engine
- *
  * SUBTEST: fds-forked-engine
- *
  * SUBTEST: fds-interruptible
- *
  * SUBTEST: fds-interruptible-all
- *
  * SUBTEST: fds-interruptible-engine
- *
  * SUBTEST: fds-priority-engine
- *
  * SUBTEST: fds-sync
- *
  * SUBTEST: fds-sync-all
- *
  * SUBTEST: fds-sync-engine
- *
  * SUBTEST: forked-engine
- *
  * SUBTEST: hang-chain
- *
  * SUBTEST: hang-chain-forked
- *
  * SUBTEST: hang-chain-sync
- *
  * SUBTEST: hang-contexts
- *
  * SUBTEST: hang-contexts-chain
- *
  * SUBTEST: hang-contexts-forked
- *
  * SUBTEST: hang-contexts-priority
- *
  * SUBTEST: hang-contexts-sync
- *
  * SUBTEST: hang-fds
- *
  * SUBTEST: hang-fds-chain
- *
  * SUBTEST: hang-fds-forked
- *
  * SUBTEST: hang-fds-priority
- *
  * SUBTEST: hang-fds-sync
- *
  * SUBTEST: hang-forked
- *
  * SUBTEST: hang-normal
- *
  * SUBTEST: hang-queues
- *
  * SUBTEST: hang-queues-chain
- *
  * SUBTEST: hang-queues-forked
- *
  * SUBTEST: hang-queues-priority
- *
  * SUBTEST: hang-queues-sync
- *
  * SUBTEST: hang-sync
- *
  * SUBTEST: interruptible
- *
  * SUBTEST: interruptible-all
- *
  * SUBTEST: interruptible-engine
- *
  * SUBTEST: normal-engine
- *
  * SUBTEST: queues-chain
- *
  * SUBTEST: queues-chain-all
- *
  * SUBTEST: queues-engine
- *
  * SUBTEST: queues-forked-engine
- *
  * SUBTEST: queues-interruptible
- *
  * SUBTEST: queues-interruptible-all
- *
  * SUBTEST: queues-interruptible-engine
- *
  * SUBTEST: queues-priority-engine
- *
  * SUBTEST: queues-sync
- *
  * SUBTEST: queues-sync-all
- *
  * SUBTEST: queues-sync-engine
- *
  * SUBTEST: sync-engine
  */
 
@@ -471,6 +294,7 @@ static void whisper(int fd, const intel_ctx_t *ctx,
 {
 	const uint32_t bbe = MI_BATCH_BUFFER_END;
 	const unsigned int gen = intel_gen(intel_get_drm_devid(fd));
+	const unsigned int ncpus = sysconf(_SC_NPROCESSORS_ONLN);
 	struct drm_i915_gem_exec_object2 batches[QLEN];
 	struct drm_i915_gem_relocation_entry inter[QLEN];
 	struct drm_i915_gem_relocation_entry reloc;
@@ -483,18 +307,19 @@ static void whisper(int fd, const intel_ctx_t *ctx,
 	int fds[64];
 	intel_ctx_cfg_t local_cfg;
 	const intel_ctx_t *contexts[64];
-	unsigned nengine;
 	uint32_t batch[16];
 	unsigned int relocations = 0;
 	unsigned int reloc_migrations = 0;
 	unsigned int reloc_interruptions = 0;
 	unsigned int eb_migrations = 0;
 	struct power_sample sample[2];
+	unsigned int nengine;
+	unsigned int nchild;
+	unsigned int qlen;
 	struct igt_power gpu;
 	uint64_t old_offset;
 	int i, n, loc;
 	int debugfs;
-	int nchild;
 	bool has_relocs = gem_has_relocations(fd);
 
 	if (flags & PRIORITY) {
@@ -533,9 +358,13 @@ static void whisper(int fd, const intel_ctx_t *ctx,
 
 	nchild = 1;
 	if (flags & FORKED)
-		nchild *= sysconf(_SC_NPROCESSORS_ONLN);
+		nchild *= ncpus;
 	if (flags & ALL)
 		nchild *= nengine;
+	nchild = min(nchild, 2 * max(ncpus, nengine));
+	qlen = max(2u, QLEN * nengine / (nchild + nengine - 1));
+	igt_info("Using nchild:%d (out of ncpus:%d and nengine:%d), with a qlen:%d\n",
+		 nchild, ncpus, nengine, qlen);
 
 	intel_detect_and_clear_missed_interrupts(fd);
 	igt_power_get_energy(&gpu, &sample[0]);
@@ -625,19 +454,19 @@ static void whisper(int fd, const intel_ctx_t *ctx,
 		}
 
 		memset(batches, 0, sizeof(batches));
-		for (n = 0; n < QLEN; n++) {
+		for (n = 0; n < qlen; n++) {
 			batches[n].handle = gem_create(fd, 4096);
 			gem_write(fd, batches[n].handle, 0, &bbe, sizeof(bbe));
 		}
 		execbuf.buffers_ptr = to_user_pointer(batches);
-		execbuf.buffer_count = QLEN;
+		execbuf.buffer_count = qlen;
 		gem_execbuf(fd, &execbuf);
 
 		execbuf.buffers_ptr = to_user_pointer(tmp);
 		execbuf.buffer_count = 2;
 
 		old_offset = store.offset;
-		for (n = 0; n < QLEN; n++) {
+		for (n = 0; n < qlen; n++) {
 			if (gen >= 8) {
 				batch[1] = old_offset + loc;
 				batch[2] = (old_offset + loc) >> 32;
@@ -702,8 +531,8 @@ static void whisper(int fd, const intel_ctx_t *ctx,
 					gem_sync(fd, tmp[0].handle);
 				scratch = tmp[0];
 
-				gem_write(fd, batches[QLEN-1].handle, loc, &pass, sizeof(pass));
-				for (n = QLEN; --n >= 1; ) {
+				gem_write(fd, batches[qlen-1].handle, loc, &pass, sizeof(pass));
+				for (n = qlen; --n >= 1; ) {
 					uint32_t handle[2] = {};
 					int this_fd = fd;
 
@@ -825,7 +654,7 @@ static void whisper(int fd, const intel_ctx_t *ctx,
 				gem_vm_destroy(fd, local_cfg.vm);
 			}
 		}
-		for (n = 0; n < QLEN; n++)
+		for (n = 0; n < qlen; n++)
 			gem_close(fd, batches[n].handle);
 		if (flags & FDS) {
 			for (n = 0; n < 64; n++)
