@@ -10,81 +10,64 @@
 /**
  * TEST: gem pxp
  * Description: Test PXP that manages protected content through arbitrated HW-PXP-session
- * Run type: FULL
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: driver
+ * Functionality: pxp
+ * Feature: Protected Xe Path
  *
  * SUBTEST: create-protected-buffer
- * Feature: PXP
  *
  * SUBTEST: create-regular-buffer
  * Description: Verify protected buffer on supported hw:
- * Feature: PXP
  *
  * SUBTEST: create-regular-context-1
  * Description: Verify protected context on supported hw:
- * Feature: PXP
  *
  * SUBTEST: create-regular-context-2
- * Feature: PXP
  *
  * SUBTEST: create-valid-protected-context
- * Feature: PXP
  *
  * SUBTEST: display-protected-crc
  * Description: Test the display CRC
- * Feature: PXP
  *
  * SUBTEST: dmabuf-shared-protected-dst-is-context-refcounted
  * Feature: PXP, prime
  *
  * SUBTEST: fail-invalid-protected-context
- * Feature: PXP
  *
  * SUBTEST: hw-rejects-pxp-buffer
  * Description: Verify protected buffer on unsupported hw:
- * Feature: PXP
  *
  * SUBTEST: hw-rejects-pxp-context
  * Description: Verify protected context on unsupported hw:
- * Feature: PXP
  *
  * SUBTEST: protected-encrypted-src-copy-not-readible
- * Feature: PXP
  *
  * SUBTEST: protected-raw-src-copy-not-readible
- * Feature: PXP
  *
  * SUBTEST: regular-baseline-src-copy-readible
  * Description: Verify protected render operations:
- * Feature: PXP
  *
  * SUBTEST: reject-modify-context-protection-off-1
- * Feature: PXP
  *
  * SUBTEST: reject-modify-context-protection-off-2
- * Feature: PXP
  *
  * SUBTEST: reject-modify-context-protection-off-3
- * Feature: PXP
  *
  * SUBTEST: reject-modify-context-protection-on
  * Description: Verify protected context integrity:
- * Feature: PXP
  *
  * SUBTEST: verify-pxp-execution-after-suspend-resume
- * Feature: PXP
  *
  * SUBTEST: verify-pxp-key-change-after-suspend-resume
  * Description: Verify suspend-resume teardown management:
- * Feature: PXP
  *
  * SUBTEST: verify-pxp-stale-buf-execution
- * Feature: PXP
  *
  * SUBTEST: verify-pxp-stale-buf-optout-execution
- * Feature: PXP
  *
  * SUBTEST: verify-pxp-stale-ctx-execution
- * Feature: PXP
  */
 
 IGT_TEST_DESCRIPTION("Test PXP that manages protected content through arbitrated HW-PXP-session");
@@ -545,12 +528,20 @@ static void test_render_baseline(int i915)
 	igt_assert(ibb);
 
 	dstbo = alloc_and_fill_dest_buff(i915, false, TSTSURF_SIZE, TSTSURF_INITCOLOR1);
-	dstbuf = intel_buf_create_using_handle(bops, dstbo, TSTSURF_WIDTH, TSTSURF_HEIGHT,
-					       TSTSURF_BYTESPP*8, 0, I915_TILING_NONE, 0);
+	dstbuf = intel_buf_create_using_handle_and_size(bops, dstbo,
+							TSTSURF_WIDTH,
+							TSTSURF_HEIGHT,
+							TSTSURF_BYTESPP*8, 0,
+							I915_TILING_NONE, 0,
+							TSTSURF_SIZE);
 
 	srcbo = alloc_and_fill_dest_buff(i915, false, TSTSURF_SIZE, TSTSURF_FILLCOLOR1);
-	srcbuf = intel_buf_create_using_handle(bops, srcbo, TSTSURF_WIDTH, TSTSURF_HEIGHT,
-					       TSTSURF_BYTESPP*8, 0, I915_TILING_NONE, 0);
+	srcbuf = intel_buf_create_using_handle_and_size(bops, srcbo,
+							TSTSURF_WIDTH,
+							TSTSURF_HEIGHT,
+							TSTSURF_BYTESPP*8, 0,
+							I915_TILING_NONE, 0,
+							TSTSURF_SIZE);
 
 	render_copy(ibb, srcbuf, 0, 0, TSTSURF_WIDTH, TSTSURF_HEIGHT, dstbuf, 0, 0);
 	gem_sync(i915, dstbo);
@@ -595,13 +586,21 @@ static void __test_render_pxp_src_to_protdest(int i915, uint32_t *outpixels, int
 	intel_bb_set_pxp(ibb, true, DISPLAY_APPTYPE, I915_PROTECTED_CONTENT_DEFAULT_SESSION);
 
 	dstbo = alloc_and_fill_dest_buff(i915, true, TSTSURF_SIZE, TSTSURF_INITCOLOR2);
-	dstbuf = intel_buf_create_using_handle(bops, dstbo, TSTSURF_WIDTH, TSTSURF_HEIGHT,
-						TSTSURF_BYTESPP*8, 0, I915_TILING_NONE, 0);
+	dstbuf = intel_buf_create_using_handle_and_size(bops, dstbo,
+							TSTSURF_WIDTH,
+							TSTSURF_HEIGHT,
+							TSTSURF_BYTESPP*8, 0,
+							I915_TILING_NONE, 0,
+							TSTSURF_SIZE);
 	intel_buf_set_pxp(dstbuf, true);
 
 	srcbo = alloc_and_fill_dest_buff(i915, false, TSTSURF_SIZE, TSTSURF_FILLCOLOR2);
-	srcbuf = intel_buf_create_using_handle(bops, srcbo, TSTSURF_WIDTH, TSTSURF_HEIGHT,
-						TSTSURF_BYTESPP*8, 0, I915_TILING_NONE, 0);
+	srcbuf = intel_buf_create_using_handle_and_size(bops, srcbo,
+							TSTSURF_WIDTH,
+							TSTSURF_HEIGHT,
+							TSTSURF_BYTESPP*8, 0,
+							I915_TILING_NONE, 0,
+							TSTSURF_SIZE);
 
 	render_copy(ibb, srcbuf, 0, 0, TSTSURF_WIDTH, TSTSURF_HEIGHT, dstbuf, 0, 0);
 	gem_sync(i915, dstbo);
@@ -656,13 +655,21 @@ static void test_render_pxp_protsrc_to_protdest(int i915)
 	intel_bb_set_pxp(ibb, true, DISPLAY_APPTYPE, I915_PROTECTED_CONTENT_DEFAULT_SESSION);
 
 	dstbo = alloc_and_fill_dest_buff(i915, true, TSTSURF_SIZE, TSTSURF_INITCOLOR2);
-	dstbuf = intel_buf_create_using_handle(bops, dstbo, TSTSURF_WIDTH, TSTSURF_HEIGHT,
-						TSTSURF_BYTESPP*8, 0, I915_TILING_NONE, 0);
+	dstbuf = intel_buf_create_using_handle_and_size(bops, dstbo,
+							TSTSURF_WIDTH,
+							TSTSURF_HEIGHT,
+							TSTSURF_BYTESPP*8, 0,
+							I915_TILING_NONE, 0,
+							TSTSURF_SIZE);
 	intel_buf_set_pxp(dstbuf, true);
 
 	srcbo = alloc_and_fill_dest_buff(i915, false, TSTSURF_SIZE, TSTSURF_FILLCOLOR2);
-	srcbuf = intel_buf_create_using_handle(bops, srcbo, TSTSURF_WIDTH, TSTSURF_HEIGHT,
-						TSTSURF_BYTESPP*8, 0, I915_TILING_NONE, 0);
+	srcbuf = intel_buf_create_using_handle_and_size(bops, srcbo,
+							TSTSURF_WIDTH,
+							TSTSURF_HEIGHT,
+							TSTSURF_BYTESPP*8, 0,
+							I915_TILING_NONE, 0,
+							TSTSURF_SIZE);
 
 	render_copy(ibb, srcbuf, 0, 0, TSTSURF_WIDTH, TSTSURF_HEIGHT, dstbuf, 0, 0);
 	gem_sync(i915, dstbo);
@@ -682,8 +689,12 @@ static void test_render_pxp_protsrc_to_protdest(int i915)
 				TSTSURF_SIZE, 0, encrypted, TSTSURF_SIZE);
 
 	dstbo2 = alloc_and_fill_dest_buff(i915, true, TSTSURF_SIZE, TSTSURF_INITCOLOR3);
-	dstbuf2 = intel_buf_create_using_handle(bops, dstbo2, TSTSURF_WIDTH, TSTSURF_HEIGHT,
-						TSTSURF_BYTESPP*8, 0, I915_TILING_NONE, 0);
+	dstbuf2 = intel_buf_create_using_handle_and_size(bops, dstbo2,
+							 TSTSURF_WIDTH,
+							 TSTSURF_HEIGHT,
+							 TSTSURF_BYTESPP*8, 0,
+							 I915_TILING_NONE, 0,
+							 TSTSURF_SIZE);
 	intel_buf_set_pxp(dstbuf2, true);
 	intel_buf_set_pxp(dstbuf, true);/*this time, src is protected*/
 
@@ -751,15 +762,27 @@ static void test_pxp_dmabuffshare_refcnt(int i915)
 		if (n == 1)
 			fill_bo_content(fd[1], dbo[1], TSTSURF_SIZE, TSTSURF_INITCOLOR2);
 
-		dbuf[n] = intel_buf_create_using_handle(bops[n], dbo[n], TSTSURF_WIDTH,
-							TSTSURF_HEIGHT,	TSTSURF_BYTESPP*8, 0,
-							I915_TILING_NONE, 0);
+		dbuf[n] = intel_buf_create_using_handle_and_size(bops[n],
+								 dbo[n],
+								 TSTSURF_WIDTH,
+								 TSTSURF_HEIGHT,
+								 TSTSURF_BYTESPP*8,
+								 0,
+								 I915_TILING_NONE,
+								 0,
+								 TSTSURF_SIZE);
 		intel_buf_set_pxp(dbuf[n], true);
 
 		sbo[n] = alloc_and_fill_dest_buff(fd[n], false, TSTSURF_SIZE, TSTSURF_FILLCOLOR1);
-		sbuf[n] = intel_buf_create_using_handle(bops[n], sbo[n], TSTSURF_WIDTH,
-							TSTSURF_HEIGHT, TSTSURF_BYTESPP*8, 0,
-							I915_TILING_NONE, 0);
+		sbuf[n] = intel_buf_create_using_handle_and_size(bops[n],
+								 sbo[n],
+								 TSTSURF_WIDTH,
+								 TSTSURF_HEIGHT,
+								 TSTSURF_BYTESPP*8,
+								 0,
+								 I915_TILING_NONE,
+								 0,
+								 TSTSURF_SIZE);
 
 		render_copy(ibb[n], sbuf[n], 0, 0, TSTSURF_WIDTH, TSTSURF_HEIGHT,
 			    dbuf[n], 0, 0);
@@ -788,7 +811,7 @@ static void test_pxp_dmabuffshare_refcnt(int i915)
 	for (n = 0; n < (TSTSURF_SIZE/4); ++n)
 		if (encrypted[0][n] == encrypted[1][n])
 			++num_matches;
-	igt_assert(num_matches == (TSTSURF_SIZE/4));
+	igt_assert_eq(num_matches, (TSTSURF_SIZE / 4));
 }
 
 
@@ -841,7 +864,7 @@ static void emit_pipectrl(struct intel_bb *ibb, struct intel_buf *fenceb, bool b
 	uint32_t pipe_ctl_flags = 0;
 	uint32_t ps_op_id;
 
-	intel_bb_out(ibb, GFX_OP_PIPE_CONTROL(4));
+	intel_bb_out(ibb, GFX_OP_PIPE_CONTROL(2));
 	intel_bb_out(ibb, pipe_ctl_flags);
 
 	if (before)
@@ -913,8 +936,11 @@ static void prepare_exec_assets(int i915, struct simple_exec_assets *data, bool
 	data->bops = buf_ops_create(i915);
 	igt_assert(data->bops);
 
-	data->fencebuf = intel_buf_create_using_handle(data->bops, data->fencebo, 256, 4,
-						       32, 0, I915_TILING_NONE, 0);
+	data->fencebuf = intel_buf_create_using_handle_and_size(data->bops,
+								data->fencebo,
+								256, 4, 32, 0,
+								I915_TILING_NONE,
+								0, 4096);
 	intel_bb_add_intel_buf(data->ibb, data->fencebuf, true);
 }
 
@@ -962,7 +988,7 @@ static void test_pxp_stale_ctx_execution(int i915)
 	 */
 	prepare_exec_assets(i915, &data, true, false);
 	ret = gem_execbuf_flush_store_dw(i915, data.ibb, data.ctx, data.fencebuf);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	trigger_pxp_debugfs_forced_teardown(i915);
 
@@ -982,7 +1008,7 @@ static void test_pxp_stale_buf_execution(int i915)
 	/* Use pxp buffers with pxp context for testing for invalidation of protected buffers. */
 	prepare_exec_assets(i915, &data, true, true);
 	ret = gem_execbuf_flush_store_dw(i915, data.ibb, data.ctx, data.fencebuf);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	trigger_pxp_debugfs_forced_teardown(i915);
 
@@ -1029,7 +1055,7 @@ static void test_pxp_stale_buf_optout_execution(int i915)
 	 * the intent of the subtest) to ensure ARB session is alive.
 	 */
 	ret = create_ctx_with_params(i915, true, true, true, false, &tmpctx);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/*
 	 * Use a normal context for testing opt-out behavior
@@ -1037,7 +1063,7 @@ static void test_pxp_stale_buf_optout_execution(int i915)
 	 */
 	prepare_exec_assets(i915, &data, false, true);
 	ret = gem_execbuf_flush_store_dw(i915, data.ibb, data.ctx, data.fencebuf);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	trigger_pxp_debugfs_forced_teardown(i915);
 
@@ -1062,7 +1088,7 @@ static void test_pxp_pwrcycle_staleasset_execution(int i915, struct powermgt_dat
 	 */
 	prepare_exec_assets(i915, &data[0], true, false);
 	ret = gem_execbuf_flush_store_dw(i915, data[0].ibb, data[0].ctx, data[0].fencebuf);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/*
 	 * For asset data[1]: Use pxp buffers with pxp context for testing for invalidation
@@ -1070,7 +1096,7 @@ static void test_pxp_pwrcycle_staleasset_execution(int i915, struct powermgt_dat
 	 */
 	prepare_exec_assets(i915, &data[1], true, true);
 	ret = gem_execbuf_flush_store_dw(i915, data[1].ibb, data[1].ctx, data[1].fencebuf);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/*
 	 * For asset data[2]: Use a normal context for testing opt-out behavior
@@ -1078,7 +1104,7 @@ static void test_pxp_pwrcycle_staleasset_execution(int i915, struct powermgt_dat
 	 */
 	prepare_exec_assets(i915, &data[2], false, true);
 	ret = gem_execbuf_flush_store_dw(i915, data[2].ibb, data[2].ctx, data[2].fencebuf);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/* Do an S3 suspend resume cycle which also causes the pxp teardown event */
 	trigger_powermgt_suspend_cycle(i915, pm);
@@ -1144,15 +1170,19 @@ static void setup_protected_fb(int i915, int width, int height, igt_fb_t *fb, ui
 			      fb->modifier, fb->strides, fb->offsets, fb->num_planes,
 			      DRM_MODE_FB_MODIFIERS, &fb->fb_id));
 
-	dstbuf = intel_buf_create_using_handle(bops, fb->gem_handle, fb->width, fb->height,
-					       fb->plane_bpp[0], 0,
-					       igt_fb_mod_to_tiling(fb->modifier), 0);
+	dstbuf = intel_buf_create_using_handle_and_size(bops, fb->gem_handle,
+							fb->width, fb->height,
+							fb->plane_bpp[0], 0,
+							igt_fb_mod_to_tiling(fb->modifier),
+							0, fb->size);
 	dstbuf->is_protected = true;
 
 	srcbo = alloc_and_fill_dest_buff(i915, false, fb->size, TSTSURF_GREENCOLOR);
-	srcbuf = intel_buf_create_using_handle(bops, srcbo, fb->width, fb->height,
-					       fb->plane_bpp[0], 0,
-					       igt_fb_mod_to_tiling(fb->modifier), 0);
+	srcbuf = intel_buf_create_using_handle_and_size(bops, srcbo, fb->width,
+							fb->height,
+							fb->plane_bpp[0], 0,
+							igt_fb_mod_to_tiling(fb->modifier),
+							0, fb->size);
 
 	ibb = intel_bb_create_with_context(i915, ctx, 0, NULL, 4096);
 	igt_assert(ibb);
