@@ -28,126 +28,118 @@
  * Description: Test render compression (RC), in which the main surface is
  *              complemented by a color control surface (CCS) that the display
  *              uses to interpret the compressed data.
+ * Driver requirement: i915, xe
+ * Functionality: ccs, tiling
+ * Mega feature: E2E Compression
+ * Test category: functionality test
  */
+
 #include "igt.h"
 
 #include "i915/gem_create.h"
+#include "intel_pat.h"
+#include "intel_blt.h"
+#include "intel_mocs.h"
+#include "xe/xe_ioctl.h"
+#include "xe/xe_query.h"
 
 /**
- * SUBTEST: %s-%s-%s
- * Description: Test %arg[2] with given %arg[3] modifier
- * Driver requirement: i915, xe
- * Functionality: ccs, tiling
- * Mega feature: General Display Features
- * Test category: functionality test
+ * SUBTEST: %s-%s
+ * Description: Test %arg[1] with given %arg[2] modifier
  *
  * arg[1]:
  *
- * @pipe-A:                    Pipe A
- * @pipe-B:                    Pipe B
- * @pipe-C:                    Pipe C
- * @pipe-D:                    Pipe D
- * @pipe-E:                    Pipe E
- * @pipe-F:                    Pipe F
- * @pipe-G:                    Pipe G
- * @pipe-H:                    Pipe H
- *
- * arg[2]:
- *
  * @bad-aux-stride:            Bad AUX stride
  * @ccs-on-another-bo:         CCS with different BO
  * @missing-ccs-buffer:        Missing CCS buffer
  *
- * arg[3]:
+ * arg[2]:
  *
- * @y_tiled_ccs:               Y tiled ccs
- * @y_tiled_gen12_mc_ccs:      Y tiled gen12 mc ccs
- * @y_tiled_gen12_rc_ccs:      Y tiled gen12 rc ccs
- * @y_tiled_gen12_rc_ccs_cc:   Y tiled gen12 rc ccs cc
- * @yf_tiled_ccs:              YF tiled ccs
- * @4_tiled_mtl_mc_ccs:        4 tiled mtl mc ccs
- * @4_tiled_mtl_rc_ccs:        4 tiled mtl rc ccs
- * @4_tiled_mtl_rc_ccs_cc:     4 tiled mtl rc ccs cc
+ * @y-tiled-ccs:               Y tiled ccs
+ * @y-tiled-gen12-mc-ccs:      Y tiled gen12 mc ccs
+ * @y-tiled-gen12-rc-ccs:      Y tiled gen12 rc ccs
+ * @y-tiled-gen12-rc-ccs-cc:   Y tiled gen12 rc ccs cc
+ * @yf-tiled-ccs:              YF tiled ccs
+ * @4-tiled-mtl-mc-ccs:        4 tiled mtl mc ccs
+ * @4-tiled-mtl-rc-ccs:        4 tiled mtl rc ccs
+ * @4-tiled-mtl-rc-ccs-cc:     4 tiled mtl rc ccs cc
  */
 
 /**
- * SUBTEST: %s-%s-%s
- * Description: Test %arg[2] with %arg[3] modifier
- * Driver requirement: i915, xe
- * Functionality: ccs, tiling
- * Mega feature: General Display Features
- * Test category: functionality test
+ * SUBTEST: %s-%s
+ * Description: Test %arg[1] with %arg[2] modifier
  *
  * arg[1]:
  *
- * @pipe-A:                      Pipe A
- * @pipe-B:                      Pipe B
- * @pipe-C:                      Pipe C
- * @pipe-D:                      Pipe D
- * @pipe-E:                      Pipe E
- * @pipe-F:                      Pipe F
- * @pipe-G:                      Pipe G
- * @pipe-H:                      Pipe H
+ * @bad-pixel-format:            Bad pixel format
  *
  * arg[2]:
  *
- * @bad-pixel-format:            Bad pixel format
+ * @4-tiled-dg2-mc-ccs:        4 tiled mc ccs
+ * @4-tiled-dg2-rc-ccs:        4 tiled dg2 rc ccs
+ * @4-tiled-dg2-rc-ccs-cc:     4 tiled dg2 rc ccs cc
+ * @4-tiled-mtl-mc-ccs:        4 tiled mtl mc ccs
+ * @4-tiled-mtl-rc-ccs:        4 tiled mtl rc ccs
+ * @4-tiled-mtl-rc-ccs-cc:     4 tiled mtl rc ccs cc
+ * @y-tiled-ccs:               Y tiled ccs
+ * @y-tiled-gen12-mc-ccs:      Y tiled gen12 mc ccs
+ * @y-tiled-gen12-rc-ccs:      Y tiled gen12 rc ccs
+ * @y-tiled-gen12-rc-ccs-cc:   Y tiled gen12 rc ccs cc
+ * @yf-tiled-ccs:              YF tiled ccs
+ */
+
+/**
+ * SUBTEST: %s-%s
+ * Description: Test %arg[1] with %arg[2] modifier
+ *
+ * arg[1]:
+ *
  * @crc-primary-basic:           Primary plane CRC compatibility
  * @crc-sprite-planes-basic:     Sprite plane CRC compatability
  * @random-ccs-data:             Random CCS data
  *
- * arg[3]:
+ * arg[2]:
  *
- * @4_tiled_dg2_mc_ccs:        4 tiled mc ccs
- * @4_tiled_dg2_rc_ccs:        4 tiled dg2 rc ccs
- * @4_tiled_dg2_rc_ccs_cc:     4 tiled dg2 rc ccs cc
- * @4_tiled_mtl_mc_ccs:        4 tiled mtl mc ccs
- * @4_tiled_mtl_rc_ccs:        4 tiled mtl rc ccs
- * @4_tiled_mtl_rc_ccs_cc:     4 tiled mtl rc ccs cc
- * @y_tiled_ccs:               Y tiled ccs
- * @y_tiled_gen12_mc_ccs:      Y tiled gen12 mc ccs
- * @y_tiled_gen12_rc_ccs:      Y tiled gen12 rc ccs
- * @y_tiled_gen12_rc_ccs_cc:   Y tiled gen12 rc ccs cc
- * @yf_tiled_ccs:              YF tiled ccs
+ * @4-tiled-bmg-ccs:           4 tiled xe2 pat controlled ccs
+ * @4-tiled-lnl-ccs:           4 tiled xe2 pat controlled ccs
+ * @4-tiled-dg2-mc-ccs:        4 tiled mc ccs
+ * @4-tiled-dg2-rc-ccs:        4 tiled dg2 rc ccs
+ * @4-tiled-dg2-rc-ccs-cc:     4 tiled dg2 rc ccs cc
+ * @4-tiled-mtl-mc-ccs:        4 tiled mtl mc ccs
+ * @4-tiled-mtl-rc-ccs:        4 tiled mtl rc ccs
+ * @4-tiled-mtl-rc-ccs-cc:     4 tiled mtl rc ccs cc
+ * @y-tiled-ccs:               Y tiled ccs
+ * @y-tiled-gen12-mc-ccs:      Y tiled gen12 mc ccs
+ * @y-tiled-gen12-rc-ccs:      Y tiled gen12 rc ccs
+ * @y-tiled-gen12-rc-ccs-cc:   Y tiled gen12 rc ccs cc
+ * @yf-tiled-ccs:              YF tiled ccs
  */
 
 /**
- * SUBTEST: %s-%s-%s
- * Description: Test %arg[2] with %arg[3] modifier
- * Driver requirement: i915, xe
+ * SUBTEST: %s-%s
+ * Description: Test %arg[1] with %arg[2] modifier
  * Functionality: ccs, rotation, tiling
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * arg[1]:
  *
- * @pipe-A:                      Pipe A
- * @pipe-B:                      Pipe B
- * @pipe-C:                      Pipe C
- * @pipe-D:                      Pipe D
- * @pipe-E:                      Pipe E
- * @pipe-F:                      Pipe F
- * @pipe-G:                      Pipe G
- * @pipe-H:                      Pipe H
- *
- * arg[2]:
- *
  * @bad-rotation-90:             90 degree rotation
  * @crc-primary-rotation-180:    180 degree rotation
  *
- * arg[3]:
+ * arg[2]:
  *
- * @4_tiled_dg2_mc_ccs:        4 tiled mc ccs
- * @4_tiled_dg2_rc_ccs:        4 tiled dg2 rc ccs
- * @4_tiled_dg2_rc_ccs_cc:     4 tiled dg2 rc ccs cc
- * @4_tiled_mtl_mc_ccs:        4 tiled mtl mc ccs
- * @4_tiled_mtl_rc_ccs:        4 tiled mtl rc ccs
- * @4_tiled_mtl_rc_ccs_cc:     4 tiled mtl rc ccs cc
- * @y_tiled_ccs:               Y tiled ccs
- * @y_tiled_gen12_mc_ccs:      Y tiled gen12 mc ccs
- * @y_tiled_gen12_rc_ccs:      Y tiled gen12 rc ccs
- * @y_tiled_gen12_rc_ccs_cc:   Y tiled gen12 rc ccs cc
- * @yf_tiled_ccs:              YF tiled ccs
+ * @4-tiled-bmg-ccs:           4 tiled xe2 pat controlled ccs
+ * @4-tiled-lnl-ccs:           4 tiled xe2 pat controlled ccs
+ * @4-tiled-dg2-mc-ccs:        4 tiled mc ccs
+ * @4-tiled-dg2-rc-ccs:        4 tiled dg2 rc ccs
+ * @4-tiled-dg2-rc-ccs-cc:     4 tiled dg2 rc ccs cc
+ * @4-tiled-mtl-mc-ccs:        4 tiled mtl mc ccs
+ * @4-tiled-mtl-rc-ccs:        4 tiled mtl rc ccs
+ * @4-tiled-mtl-rc-ccs-cc:     4 tiled mtl rc ccs cc
+ * @y-tiled-ccs:               Y tiled ccs
+ * @y-tiled-gen12-mc-ccs:      Y tiled gen12 mc ccs
+ * @y-tiled-gen12-rc-ccs:      Y tiled gen12 rc ccs
+ * @y-tiled-gen12-rc-ccs-cc:   Y tiled gen12 rc ccs cc
+ * @yf-tiled-ccs:              YF tiled ccs
  */
 
 #define SDR_PLANE_BASE	3
@@ -207,6 +199,7 @@ static const struct {
 static const uint32_t formats[] = {
 	DRM_FORMAT_XYUV8888,
 	DRM_FORMAT_XRGB8888,
+	DRM_FORMAT_XRGB2101010,
 	DRM_FORMAT_YUYV,
 	DRM_FORMAT_NV12,
 	DRM_FORMAT_P012,
@@ -217,17 +210,19 @@ static const struct {
 	uint64_t modifier;
 	const char *str;
 } ccs_modifiers[] = {
-	{I915_FORMAT_MOD_Y_TILED_CCS, "y_tiled_ccs"},
-	{I915_FORMAT_MOD_Yf_TILED_CCS, "yf_tiled_ccs"},
-	{I915_FORMAT_MOD_Y_TILED_GEN12_RC_CCS, "y_tiled_gen12_rc_ccs"},
-	{I915_FORMAT_MOD_Y_TILED_GEN12_RC_CCS_CC, "y_tiled_gen12_rc_ccs_cc"},
-	{I915_FORMAT_MOD_Y_TILED_GEN12_MC_CCS, "y_tiled_gen12_mc_ccs"},
-	{I915_FORMAT_MOD_4_TILED_DG2_RC_CCS, "4_tiled_dg2_rc_ccs"},
-	{I915_FORMAT_MOD_4_TILED_DG2_MC_CCS, "4_tiled_dg2_mc_ccs"},
-	{I915_FORMAT_MOD_4_TILED_DG2_RC_CCS_CC, "4_tiled_dg2_rc_ccs_cc"},
-	{I915_FORMAT_MOD_4_TILED_MTL_RC_CCS, "4_tiled_mtl_rc_ccs"},
-	{I915_FORMAT_MOD_4_TILED_MTL_MC_CCS, "4_tiled_mtl_mc_ccs"},
-	{I915_FORMAT_MOD_4_TILED_MTL_RC_CCS_CC, "4_tiled_mtl_rc_ccs_cc"},
+	{I915_FORMAT_MOD_Y_TILED_CCS, "y-tiled-ccs"},
+	{I915_FORMAT_MOD_Yf_TILED_CCS, "yf-tiled-ccs"},
+	{I915_FORMAT_MOD_Y_TILED_GEN12_RC_CCS, "y-tiled-gen12-rc-ccs"},
+	{I915_FORMAT_MOD_Y_TILED_GEN12_RC_CCS_CC, "y-tiled-gen12-rc-ccs-cc"},
+	{I915_FORMAT_MOD_Y_TILED_GEN12_MC_CCS, "y-tiled-gen12-mc-ccs"},
+	{I915_FORMAT_MOD_4_TILED_DG2_RC_CCS, "4-tiled-dg2-rc-ccs"},
+	{I915_FORMAT_MOD_4_TILED_DG2_MC_CCS, "4-tiled-dg2-mc-ccs"},
+	{I915_FORMAT_MOD_4_TILED_DG2_RC_CCS_CC, "4-tiled-dg2-rc-ccs-cc"},
+	{I915_FORMAT_MOD_4_TILED_MTL_RC_CCS, "4-tiled-mtl-rc-ccs"},
+	{I915_FORMAT_MOD_4_TILED_MTL_MC_CCS, "4-tiled-mtl-mc-ccs"},
+	{I915_FORMAT_MOD_4_TILED_MTL_RC_CCS_CC, "4-tiled-mtl-rc-ccs-cc"},
+	{I915_FORMAT_MOD_4_TILED_BMG_CCS, "4-tiled-bmg-ccs"},
+	{I915_FORMAT_MOD_4_TILED_LNL_CCS, "4-tiled-lnl-ccs"},
 };
 
 static bool check_ccs_planes;
@@ -280,7 +275,7 @@ create_fb_prepare_add(int drm_fd, int width, int height,
 		      igt_fb_t *fb, struct drm_mode_fb_cmd2 *f)
 {
 	igt_create_bo_for_fb(drm_fd, width, height, format, modifier, fb);
-	igt_assert(fb->gem_handle > 0);
+	igt_assert_lt_u32(0, fb->gem_handle);
 
 	addfb_init(fb, f);
 }
@@ -310,17 +305,19 @@ static void check_ccs_plane(int drm_fd, igt_fb_t *fb, int plane)
 	ccs_size = fb->strides[plane] * fb->plane_height[plane];
 	igt_assert(ccs_size);
 
-	gem_set_domain(drm_fd, fb->gem_handle, I915_GEM_DOMAIN_CPU, 0);
-
-	map = gem_mmap__cpu(drm_fd, fb->gem_handle, 0, fb->size, PROT_READ);
-
+	if (is_i915_device(drm_fd)) {
+		gem_set_domain(drm_fd, fb->gem_handle, I915_GEM_DOMAIN_CPU, 0);
+		map = gem_mmap__cpu(drm_fd, fb->gem_handle, 0, fb->size, PROT_READ);
+	} else {
+		map = xe_bo_mmap_ext(drm_fd, fb->gem_handle, fb->size, PROT_READ);
+	}
 	ccs_size = fb->strides[plane] * fb->plane_height[plane];
 	ccs_p = map + fb->offsets[plane];
 	for (i = 0; i < ccs_size; i += sizeof(uint32_t))
 		if (*(uint32_t *)(ccs_p + i))
 			break;
 
-	munmap(map, fb->size);
+	igt_assert_eq(0, gem_munmap(map, fb->size));
 
 	igt_assert_f(i < ccs_size,
 		     "CCS plane %d (for main plane %d) lacks compression meta-data\n",
@@ -336,9 +333,12 @@ static void check_ccs_cc_plane(int drm_fd, igt_fb_t *fb, int plane, const float
 	void *map;
 	uint32_t native_color;
 
-	gem_set_domain(drm_fd, fb->gem_handle, I915_GEM_DOMAIN_CPU, 0);
-
-	map = gem_mmap__cpu(drm_fd, fb->gem_handle, 0, fb->size, PROT_READ);
+	if (is_i915_device(drm_fd)) {
+		gem_set_domain(drm_fd, fb->gem_handle, I915_GEM_DOMAIN_CPU, 0);
+		map = gem_mmap__cpu(drm_fd, fb->gem_handle, 0, fb->size, PROT_READ);
+	} else {
+		map = xe_bo_mmap_ext(drm_fd, fb->gem_handle, fb->size, PROT_READ);
+	}
 	cc_p = map + fb->offsets[plane];
 
 	igt_assert(cc_color[0] == cc_p[0].f &&
@@ -351,9 +351,9 @@ static void check_ccs_cc_plane(int drm_fd, igt_fb_t *fb, int plane, const float
 		       (uint8_t)(cc_color[1] * 0xff) << 8 |
 		       (uint8_t)(cc_color[2] * 0xff);
 
-	igt_assert(native_color == cc_p[4].d);
+	igt_assert_eq_u32(native_color, cc_p[4].d);
 
-	munmap(map, fb->size);
+	igt_assert_eq(0, gem_munmap(map, fb->size));
 };
 
 static void check_all_ccs_planes(int drm_fd, igt_fb_t *fb, const float *cc_color, bool check_cc_plane)
@@ -369,20 +369,98 @@ static void check_all_ccs_planes(int drm_fd, igt_fb_t *fb, const float *cc_color
 	}
 }
 
+static void access_flat_ccs_surface(struct igt_fb *fb, bool verify_compression)
+{
+	uint64_t bb_size, ccssize = fb->size / CCS_RATIO(fb->fd);
+	uint64_t ccs_bo_size = ALIGN(ccssize, xe_get_default_alignment(fb->fd));
+	struct blt_ctrl_surf_copy_data surf = {};
+	uint32_t sysmem = system_memory(fb->fd);
+	uint32_t bb1, ccs, *ccsmap;
+	uint16_t cpu_caching = DRM_XE_GEM_CPU_CACHING_WC;
+	uint8_t uc_mocs = intel_get_uc_mocs_index(fb->fd);
+	uint8_t comp_pat_index = intel_get_pat_idx_wt(fb->fd);
+	uint32_t region = (AT_LEAST_GEN(intel_get_drm_devid(fb->fd), 20) &
+						xe_has_vram(fb->fd)) ? REGION_LMEM(0) : REGION_SMEM;
+
+	struct drm_xe_engine_class_instance inst = {
+		.engine_class = DRM_XE_ENGINE_CLASS_COPY,
+	};
+
+	uint64_t surf_ahnd;
+	uint32_t vm, exec_queue;
+	intel_ctx_t *surf_ctx;
+
+	vm = xe_vm_create(fb->fd, 0, 0);
+	exec_queue = xe_exec_queue_create(fb->fd, vm, &inst, 0);
+	surf_ctx = intel_ctx_xe(fb->fd, vm, exec_queue, 0, 0, 0);
+	surf_ahnd = intel_allocator_open(fb->fd, surf_ctx->vm,
+						INTEL_ALLOCATOR_RELOC);
+
+	ccs = xe_bo_create_caching(fb->fd, 0, ccs_bo_size, sysmem, 0, cpu_caching);
+
+	blt_ctrl_surf_copy_init(fb->fd, &surf);
+	blt_set_ctrl_surf_object(&surf.src, fb->gem_handle, region, fb->size,
+				 uc_mocs, comp_pat_index, BLT_INDIRECT_ACCESS);
+	blt_set_ctrl_surf_object(&surf.dst, ccs, sysmem, ccssize, uc_mocs,
+				 DEFAULT_PAT_INDEX, DIRECT_ACCESS);
+
+	bb_size = xe_bb_size(fb->fd, SZ_4K);
+	bb1 = xe_bo_create(fb->fd, 0, bb_size, sysmem, 0);
+	blt_set_batch(&surf.bb, bb1, bb_size, sysmem);
+	blt_ctrl_surf_copy(fb->fd, surf_ctx, NULL, surf_ahnd, &surf);
+	intel_ctx_xe_sync(surf_ctx, true);
+
+	ccsmap = xe_bo_map(fb->fd, ccs, surf.dst.size);
+
+	if (verify_compression) {
+		for (int i = 0; i < surf.dst.size / sizeof(uint32_t); i++) {
+			if (ccsmap[i] != 0)
+				goto ccs_verified;
+		}
+		igt_assert_f(false, "framebuffer was not compressed!\n");
+	} else {
+		for (int i = 0; i < surf.dst.size / sizeof(uint32_t); i++)
+			ccsmap[i] = rand();
+	}
+
+	blt_set_ctrl_surf_object(&surf.src, ccs, sysmem, ccssize,
+				 uc_mocs, DEFAULT_PAT_INDEX, DIRECT_ACCESS);
+	blt_set_ctrl_surf_object(&surf.dst, fb->gem_handle, region, fb->size,
+				 uc_mocs, comp_pat_index, INDIRECT_ACCESS);
+	blt_ctrl_surf_copy(fb->fd, surf_ctx, NULL, surf_ahnd, &surf);
+	intel_ctx_xe_sync(surf_ctx, true);
+
+ccs_verified:
+	munmap(ccsmap, ccssize);
+	gem_close(fb->fd, ccs);
+	gem_close(fb->fd, bb1);
+	xe_exec_queue_destroy(fb->fd, exec_queue);
+	xe_vm_destroy(fb->fd, vm);
+	free(surf_ctx);
+	put_ahnd(surf_ahnd);
+}
+
 static void fill_fb_random(int drm_fd, igt_fb_t *fb)
 {
 	void *map;
 	uint8_t *p;
 	int i;
 
-	gem_set_domain(drm_fd, fb->gem_handle, I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
-
-	p = map = gem_mmap__cpu(drm_fd, fb->gem_handle, 0, fb->size, PROT_WRITE);
+	if (is_i915_device(drm_fd)) {
+		gem_set_domain(drm_fd, fb->gem_handle, I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
+		p = map = gem_mmap__cpu(drm_fd, fb->gem_handle, 0, fb->size, PROT_WRITE);
+	} else {
+		p = map = xe_bo_mmap_ext(drm_fd, fb->gem_handle, fb->size, PROT_WRITE);
+	}
 
 	for (i = 0; i < fb->size; i++)
 		p[i] = rand();
 
-	munmap(map, fb->size);
+	igt_assert_eq(0, gem_munmap(map, fb->size));
+
+	/* randomize also ccs surface on Xe2 */
+	if (AT_LEAST_GEN(intel_get_drm_devid(drm_fd), 20))
+		access_flat_ccs_surface(fb, false);
 }
 
 static void test_bad_ccs_plane(data_t *data, int width, int height, int ccs_plane,
@@ -422,7 +500,11 @@ static void test_bad_ccs_plane(data_t *data, int width, int height, int ccs_plan
 
 	/* Put the CCS buffer on a different BO. */
 	if (data->flags & TEST_BAD_CCS_HANDLE) {
-		bad_ccs_bo = gem_create(data->drm_fd, fb.size);
+		bad_ccs_bo = is_i915_device(data->drm_fd) ?
+				gem_create(data->drm_fd, fb.size) :
+				xe_bo_create(data->drm_fd, 0, fb.size,
+					     vram_if_possible(data->drm_fd, 0),
+					     DRM_XE_GEM_CREATE_FLAG_NEEDS_VISIBLE_VRAM);
 		f.handles[ccs_plane] = bad_ccs_bo;
 	}
 
@@ -490,8 +572,9 @@ static void fast_clear_fb(int drm_fd, struct igt_fb *fb, const float *cc_color)
 	struct buf_ops *bops = buf_ops_create(drm_fd);
 	struct intel_buf *dst = igt_fb_create_intel_buf(drm_fd, bops, fb, "fast clear dst");
 
-	gem_set_domain(drm_fd, fb->gem_handle,
-		       I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+	if (is_i915_device(drm_fd))
+		gem_set_domain(drm_fd, fb->gem_handle,
+			       I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
 
 	fast_clear(ibb, dst, 0, 0, fb->width, fb->height, cc_color);
 
@@ -501,6 +584,146 @@ static void fast_clear_fb(int drm_fd, struct igt_fb *fb, const float *cc_color)
 	buf_ops_destroy(bops);
 }
 
+static struct blt_copy_object *blt_fb_init(const struct igt_fb *fb,
+					   uint32_t plane, uint32_t memregion,
+					   uint8_t pat_index)
+{
+	uint32_t name, handle;
+	struct blt_copy_object *blt;
+	uint64_t stride;
+
+	blt = malloc(sizeof(*blt));
+	igt_assert(blt);
+
+	name = gem_flink(fb->fd, fb->gem_handle);
+	handle = gem_open(fb->fd, name);
+
+	stride = fb->strides[plane] / 4;
+
+	blt_set_object(blt, handle, fb->size, memregion,
+		       intel_get_uc_mocs_index(fb->fd),
+		       pat_index,
+		       T_TILE4,
+		       COMPRESSION_DISABLED,
+		       COMPRESSION_TYPE_3D);
+
+	blt_set_geom(blt, stride, 0, 0, fb->width, fb->plane_height[plane], 0, 0);
+	blt->plane_offset = fb->offsets[plane];
+	blt->ptr = xe_bo_mmap_ext(fb->fd, handle, fb->size,
+				  PROT_READ | PROT_WRITE);
+	return blt;
+}
+
+static enum blt_color_depth blt_get_bpp(const struct igt_fb *fb)
+{
+	switch (fb->plane_bpp[0]) {
+	case 8:
+		return CD_8bit;
+	case 16:
+		return CD_16bit;
+	case 32:
+		return CD_32bit;
+	case 64:
+		return CD_64bit;
+	case 96:
+		return CD_96bit;
+	case 128:
+		return CD_128bit;
+	default:
+		igt_assert(0);
+	}
+}
+
+static uint32_t blt_compression_format(struct blt_copy_data *blt,
+				       const struct igt_fb *fb)
+{
+	switch (fb->drm_format) {
+	case DRM_FORMAT_XRGB8888:
+	case DRM_FORMAT_XRGB2101010:
+	case DRM_FORMAT_P010:
+	case DRM_FORMAT_P012:
+	case DRM_FORMAT_P016:
+	case DRM_FORMAT_YUYV:
+		return 8;
+	case DRM_FORMAT_XYUV8888:
+	case DRM_FORMAT_NV12:
+		return 9;
+	default:
+		igt_assert_f(0, "Unknown format\n");
+	}
+}
+
+static void xe2_ccs_blit(data_t *data, struct igt_fb *fb, struct igt_fb *temp_fb)
+{
+	uint64_t ahnd = 0;
+
+	struct blt_copy_data blt = {};
+	struct blt_copy_object *src, *dst;
+	struct blt_block_copy_data_ext ext = {}, *pext = NULL;
+	uint32_t mem_region;
+	intel_ctx_t *xe_ctx;
+	uint32_t vm, exec_queue;
+	uint32_t xe_bb;
+	uint64_t bb_size = 4096;
+	struct igt_fb *dst_fb = fb, *src_fb = temp_fb;
+
+	struct drm_xe_engine_class_instance inst = {
+		.engine_class = DRM_XE_ENGINE_CLASS_COPY,
+	};
+
+	vm = xe_vm_create(src_fb->fd, 0, 0);
+
+	exec_queue = xe_exec_queue_create(src_fb->fd, vm, &inst, 0);
+	xe_ctx = intel_ctx_xe(src_fb->fd, vm, exec_queue, 0, 0, 0);
+	mem_region = vram_if_possible(src_fb->fd, 0);
+
+	ahnd = intel_allocator_open_full(src_fb->fd, xe_ctx->vm, 0, 0,
+						INTEL_ALLOCATOR_SIMPLE,
+						ALLOC_STRATEGY_LOW_TO_HIGH, 0);
+
+	bb_size = ALIGN(bb_size + xe_cs_prefetch_size(src_fb->fd),
+			xe_get_default_alignment(src_fb->fd));
+	xe_bb = xe_bo_create(src_fb->fd, 0, bb_size,
+			     vram_if_possible(dst_fb->fd, 0), 0);
+
+	for (int i = 0; i < dst_fb->num_planes; i++) {
+		src = blt_fb_init(src_fb, i, mem_region, intel_get_pat_idx_uc(src_fb->fd));
+		dst = blt_fb_init(dst_fb, i, mem_region, intel_get_pat_idx_wt(dst_fb->fd));
+
+		blt_copy_init(src_fb->fd, &blt);
+		blt.color_depth = blt_get_bpp(src_fb);
+		blt_set_copy_object(&blt.src, src);
+		blt_set_copy_object(&blt.dst, dst);
+
+		blt_set_object_ext(&ext.src,
+				blt_compression_format(&blt, src_fb),
+				src_fb->width, src_fb->height,
+				SURFACE_TYPE_2D);
+
+		blt_set_object_ext(&ext.dst,
+				blt_compression_format(&blt, dst_fb),
+				dst_fb->width, dst_fb->height,
+				SURFACE_TYPE_2D);
+
+		pext = &ext;
+
+		blt_set_batch(&blt.bb, xe_bb, bb_size, mem_region);
+
+		blt_block_copy(src_fb->fd, xe_ctx, NULL, ahnd, &blt, pext);
+
+		blt_destroy_object(src_fb->fd, src);
+		blt_destroy_object(dst_fb->fd, dst);
+	}
+
+	put_ahnd(ahnd);
+	gem_close(dst_fb->fd, xe_bb);
+	xe_exec_queue_destroy(dst_fb->fd, exec_queue);
+	xe_vm_destroy(dst_fb->fd, vm);
+	free(xe_ctx);
+
+	access_flat_ccs_surface(fb, true);
+}
+
 static void generate_fb(data_t *data, struct igt_fb *fb,
 			int width, int height,
 			enum test_fb_flags fb_flags)
@@ -508,7 +731,6 @@ static void generate_fb(data_t *data, struct igt_fb *fb,
 	struct drm_mode_fb_cmd2 f = {0};
 	uint64_t modifier;
 	cairo_t *cr;
-	int ret;
 	const float cc_color[4] = {colors[!!data->plane].r,
 				   colors[!!data->plane].g,
 				   colors[!!data->plane].b,
@@ -539,20 +761,37 @@ static void generate_fb(data_t *data, struct igt_fb *fb,
 		if (do_fast_clear && (fb_flags & FB_COMPRESSED)) {
 			fast_clear_fb(data->drm_fd, fb, cc_color);
 		} else {
-			cr = igt_get_cairo_ctx(data->drm_fd, fb);
+			if (modifier == I915_FORMAT_MOD_4_TILED_BMG_CCS ||
+			    modifier == I915_FORMAT_MOD_4_TILED_LNL_CCS) {
+				struct igt_fb temp_fb;
+				// non compressed temporary pattern image
+				if (do_solid_fill)
+					igt_create_color_fb(data->drm_fd, width, height,
+						fb->drm_format, I915_FORMAT_MOD_4_TILED,
+						colors[c].r, colors[c].g, colors[c].b,
+						&temp_fb);
+				else
+					igt_create_pattern_fb(data->drm_fd, width, height,
+							fb->drm_format, I915_FORMAT_MOD_4_TILED,
+							&temp_fb);
 
-			if (do_solid_fill)
-				igt_paint_color(cr, 0, 0, width, height,
-						colors[c].r, colors[c].g, colors[c].b);
-			else
-				igt_paint_test_pattern(cr, width, height);
+				xe2_ccs_blit(data, fb, &temp_fb);
+				igt_remove_fb(data->drm_fd, &temp_fb);
+			} else {
+				cr = igt_get_cairo_ctx(data->drm_fd, fb);
 
-			igt_put_cairo_ctx(cr);
+				if (do_solid_fill)
+					igt_paint_color(cr, 0, 0, width, height,
+							colors[c].r, colors[c].g, colors[c].b);
+				else
+					igt_paint_test_pattern(cr, width, height);
+
+				igt_put_cairo_ctx(cr);
+			}
 		}
 	}
 
-	ret = drmIoctl(data->drm_fd, DRM_IOCTL_MODE_ADDFB2, &f);
-	igt_assert_eq(ret, 0);
+	do_ioctl(data->drm_fd, DRM_IOCTL_MODE_ADDFB2, &f);
 
 	if (check_ccs_planes)
 		check_all_ccs_planes(data->drm_fd, fb, cc_color, !(data->flags & TEST_RANDOM));
@@ -609,7 +848,8 @@ static bool try_config(data_t *data, enum test_fb_flags fb_flags,
 		return false;
 
 	if (is_ccs_cc_modifier(data->ccs_modifier) &&
-	    data->format != DRM_FORMAT_XRGB8888)
+	    data->format != DRM_FORMAT_XRGB8888 &&
+	    data->format != DRM_FORMAT_XRGB2101010)
 		return false;
 
 	if ((fb_flags & FB_MISALIGN_AUX_STRIDE) ||
@@ -715,69 +955,76 @@ static int test_ccs(data_t *data)
 
 static void test_output(data_t *data, const int testnum)
 {
-	igt_fixture {
-		bool found = false;
+	uint16_t dev_id;
 
-		data->flags = tests[testnum].flags;
+	igt_fixture
+		dev_id = intel_get_drm_devid(data->drm_fd);
 
-		for_each_valid_output_on_pipe(&data->display, data->pipe, data->output) {
-			igt_display_reset(&data->display);
-
-			igt_output_set_pipe(data->output, data->pipe);
-			if (i915_pipe_output_combo_valid(&data->display)) {
-				found = true;
-				break;
-			}
-
-		}
-		igt_require_f(found, "No valid pipe/output combo found.\n");
-	}
+	data->flags = tests[testnum].flags;
 
 	for (int i = 0; i < ARRAY_SIZE(ccs_modifiers); i++) {
-		if ((ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_DG2_RC_CCS ||
+		if (((ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_DG2_RC_CCS ||
 		    ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_DG2_MC_CCS ||
 		    ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_DG2_RC_CCS_CC) &&
-		    tests[testnum].flags & TEST_BAD_CCS_PLANE)
-		    continue;
+		    tests[testnum].flags & TEST_BAD_CCS_PLANE) ||
+		    (tests[testnum].flags & TEST_FAIL_ON_ADDFB2 &&
+		    (ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_BMG_CCS ||
+		    ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_LNL_CCS)))
+			continue;
 
 		data->ccs_modifier = ccs_modifiers[i].modifier;
 
 		igt_describe(tests[testnum].description);
-		igt_subtest_f("pipe-%s-%s-%s", kmstest_pipe_name(data->pipe),
-			      tests[testnum].testname, ccs_modifiers[i].str) {
-			int valid_tests = 0;
-
-			if (data->flags == TEST_RANDOM)
-				igt_info("Testing with seed %d\n", data->seed);
-
-			if (data->flags & TEST_ALL_PLANES) {
-				igt_display_require_output_on_pipe(&data->display, data->pipe);
-
-				for_each_plane_on_pipe(&data->display, data->pipe, data->plane) {
-					for (int j = 0; j < ARRAY_SIZE(formats); j++) {
-						data->format = formats[j];
-						valid_tests += test_ccs(data);
-					}
-				}
+		igt_subtest_with_dynamic_f("%s-%s", tests[testnum].testname, ccs_modifiers[i].str) {
+			if (ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_BMG_CCS ||
+			    ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_LNL_CCS) {
+				igt_require_f(AT_LEAST_GEN(dev_id, 20),
+					      "Xe2 platform needed.\n");
 			} else {
-				for (int j = 0; j < ARRAY_SIZE(formats); j++) {
-					data->format = formats[j];
-					valid_tests += test_ccs(data);
+				igt_require_f(intel_get_device_info(dev_id)->graphics_ver < 20,
+					      "Older than Xe2 platform needed.\n");
+			}
+
+			for_each_pipe_with_valid_output(&data->display, data->pipe, data->output) {
+				igt_display_reset(&data->display);
+
+				igt_output_set_pipe(data->output, data->pipe);
+				if (!intel_pipe_output_combo_valid(&data->display))
+					continue;
+
+				igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(data->pipe),
+							    data->output->name) {
+					int valid_tests = 0;
+
+					if (data->flags == TEST_RANDOM)
+						igt_info("Testing with seed %d\n", data->seed);
+
+					if (data->flags & TEST_ALL_PLANES) {
+						igt_display_require_output_on_pipe(&data->display, data->pipe);
+
+						for_each_plane_on_pipe(&data->display, data->pipe, data->plane) {
+							for (int j = 0; j < ARRAY_SIZE(formats); j++) {
+								data->format = formats[j];
+								valid_tests += test_ccs(data);
+							}
+						}
+					} else {
+						for (int j = 0; j < ARRAY_SIZE(formats); j++) {
+							data->format = formats[j];
+							valid_tests += test_ccs(data);
+						}
+					}
+					igt_require_f(valid_tests > 0,
+						      "no valid tests for %s on pipe %s\n",
+						      ccs_modifiers[i].str,
+						      kmstest_pipe_name(data->pipe));
 				}
 			}
-			igt_require_f(valid_tests > 0,
-				      "no valid tests for %s on pipe %s\n",
-				      ccs_modifiers[i].str,
-				      kmstest_pipe_name(data->pipe));
 		}
 	}
 
-	igt_fixture {
-		igt_output_set_pipe(data->output, PIPE_NONE);
-		igt_display_commit2(&data->display, data->display.is_atomic ?
-				    COMMIT_ATOMIC : COMMIT_LEGACY);
+	igt_fixture
 		data->plane = NULL;
-	}
 }
 
 static int opt_handler(int opt, int opt_index, void *opt_data)
@@ -808,10 +1055,8 @@ static const char *help_str =
 
 igt_main_args("cs:", NULL, help_str, opt_handler, &data)
 {
-	enum pipe pipe;
-
 	igt_fixture {
-		data.drm_fd = drm_open_driver_master(DRIVER_INTEL);
+		data.drm_fd = drm_open_driver_master(DRIVER_INTEL | DRIVER_XE);
 
 		igt_require(intel_display_ver(intel_get_drm_devid(data.drm_fd)) >= 9);
 		kmstest_set_vt_graphics_mode();
@@ -824,15 +1069,8 @@ igt_main_args("cs:", NULL, help_str, opt_handler, &data)
 			data.seed = time(NULL);
 	}
 
-	for_each_pipe_static(pipe) {
-		data.pipe = pipe;
-
-		igt_subtest_group {
-			for (int c = 0; c < ARRAY_SIZE(tests); c++) {
-				test_output(&data, c);
-			}
-		}
-	}
+	for (int c = 0; c < ARRAY_SIZE(tests); c++)
+		test_output(&data, c);
 
 	igt_fixture {
 		igt_display_fini(&data.display);
