@@ -52,203 +52,139 @@
 /**
  * TEST: perf pmu
  * Description: Test the i915 pmu perf interface
- * Run type: FULL
+ * Category: Core
+ * Mega feature: Performance interface
+ * Sub-category: Performance tests
+ * Functionality: pmu
+ * Feature: i915 pmu perf interface, pmu
+ * Test category: Perf
  *
  * SUBTEST: all-busy-check-all
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: all-busy-idle-check-all
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: busy
  * Description: Test to ensure gpu is busy when there a workload by reading engine busyness pmu counters
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: busy-accuracy-2
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: busy-accuracy-50
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: busy-accuracy-98
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: busy-check-all
  * Description: Test to ensure gpu all engines report busy when there is a workload by reading engine busyness pmu counters
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: busy-double-start
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: busy-hang
  * Description: Test to ensure there is no hanf when all engines are busy
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: busy-idle
  * Description: Test to ensure gpu engine reports idle when there is no workload
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: busy-idle-check-all
  * Description: Test to ensure gpu all engine reports idle when there is no workload
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: busy-idle-no-semaphores
  * Description: Test to verify gpu idle through engine business pmu counters
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: busy-no-semaphores
  * Description: Test to verify gpu busyness through engine business pmu counters
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: busy-start
  * Description: Test to verify gpu busyness through engine business pmu counters
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: cpu-hotplug
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: enable-race
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: event-wait
- * Feature: obsolete, pmu
+ * Feature: obsolete, pmu feature
  *
  * SUBTEST: faulting-read
- * Feature: pmu
+ * Feature: pmu feature
  *
  * SUBTEST: frequency
  * Description: Read requested freq and actual frequency via PMU within specified time interval for any given workload changes
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: gt-awake
  * Description: Setup workload on all engines,measure gt awake time via pmu
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: idle
  * Description: Test to ensure gpu is idle when there is no workload by reading engine busyness pmu counters
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: idle-no-semaphores
  * Description: Test to ensure gpu is idle when there is no workload by reading engine busyness pmu counters
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: init-busy
  * Description: Test to verify gpu busyness init through pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: init-sema
  * Description: Test to verify gpu busyness init through pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: init-wait
  * Description: Test to verify gpu busyness init through pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: interrupts
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: interrupts-sync
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: invalid-init
  * Description: Tests that i915 PMU corectly errors out in invalid initialization
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: invalid-open
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: module-unload
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: most-busy-check-all
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: most-busy-idle-check-all
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: multi-client
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: pmu-read
  * Description: Verify i915 pmu dir exists and read all events
- * Feature: pmu
- * Test category: Perf
+ * Feature: pmu feature
  *
  * SUBTEST: rc6
- * Feature: pmu
+ * Feature: pmu feature
  *
  * SUBTEST: rc6-all-gts
- * Feature: pmu
+ * Feature: pmu feature
  *
  * SUBTEST: rc6-suspend
- * Feature: pmu
+ * Feature: pmu feature
  *
  * SUBTEST: render-node-busy
- * Feature: pmu
+ * Feature: pmu feature
  *
  * SUBTEST: render-node-busy-idle
- * Feature: pmu
+ * Feature: pmu feature
  *
  * SUBTEST: semaphore-busy
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: semaphore-wait
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  *
  * SUBTEST: semaphore-wait-idle
  * Description: Test the i915 pmu perf interface
- * Feature: i915 pmu perf interface, pmu
- * Test category: Perf
  */
 
 IGT_TEST_DESCRIPTION("Test the i915 pmu perf interface");
@@ -264,7 +200,7 @@ static char *get_drpc(int i915, int gt_id)
 	int gt_dir;
 
 	gt_dir = igt_debugfs_gt_dir(i915, gt_id);
-	igt_assert(gt_dir != -1);
+	igt_assert_neq(gt_dir, -1);
 	return igt_sysfs_get(gt_dir, "drpc");
 }
 
@@ -274,7 +210,7 @@ static int open_pmu(int i915, uint64_t config)
 
 	fd = perf_i915_open(i915, config);
 	igt_skip_on(fd < 0 && errno == ENODEV);
-	igt_assert(fd >= 0);
+	igt_assert_lte(0, fd);
 
 	return fd;
 }
@@ -285,7 +221,7 @@ static int open_group(int i915, uint64_t config, int group)
 
 	fd = perf_i915_open_group(i915, config, group);
 	igt_skip_on(fd < 0 && errno == ENODEV);
-	igt_assert(fd >= 0);
+	igt_assert_lte(0, fd);
 
 	return fd;
 }
@@ -591,7 +527,7 @@ static void log_busy(unsigned int num_engines, uint64_t *val)
 		int len;
 
 		len = snprintf(p, rem, "%u=%" PRIu64 "\n",  i, val[i]);
-		igt_assert(len > 0);
+		igt_assert_lt(0, len);
 		rem -= len;
 		p += len;
 	}
@@ -1014,7 +950,7 @@ __sema_busy(int gem_fd, uint64_t ahnd, int pmu, const intel_ctx_t *ctx,
 	int timeout = 3;
 
 	/* Time spent being busy includes time waiting on semaphores */
-	igt_assert(busy_pct >= sema_pct);
+	igt_assert_lte(sema_pct, busy_pct);
 
 	gem_quiescent_gpu(gem_fd);
 
@@ -1423,7 +1359,7 @@ static void open_invalid(int i915)
 	int fd;
 
 	fd = perf_i915_open(i915, -1ULL);
-	igt_assert(fd < 0);
+	igt_assert_lt(fd, 0);
 }
 
 static bool cpu0_hotplug_support(void)
@@ -1479,7 +1415,7 @@ static void cpu_hotplug(int gem_fd)
 					       cpu), sizeof(name));
 			cpufd = open(name, O_WRONLY);
 			if (cpufd == -1) {
-				igt_assert(cpu > 0);
+				igt_assert_lt(0, cpu);
 				/*
 				 * Signal parent that we cycled through all
 				 * CPUs and we are done.
@@ -1598,7 +1534,7 @@ test_interrupts(int gem_fd)
 			close(old_fd);
 		}
 
-		igt_assert(fence_fd >= 0);
+		igt_assert_lte(0, fence_fd);
 	}
 
 	/* Wait for idle state. */
@@ -1819,9 +1755,9 @@ test_frequency(int gem_fd, unsigned int gt)
 	 */
 	__igt_sysfs_set_u32(sysfs, "rps_min_freq_mhz", min_freq);
 	__igt_sysfs_get_u32(sysfs, "rps_min_freq_mhz", &read_value);
-	if (read_value != min_freq)
-		igt_warn("Unable to restore min frequency to saved value [%u MHz], now %u MHz\n",
-			 min_freq, read_value);
+	igt_warn_on_f(read_value != min_freq,
+		      "Unable to restore min frequency to saved value [%u MHz], now %u MHz\n",
+		      min_freq, read_value);
 	close(fd[0]);
 	close(fd[1]);
 	put_ahnd(ahnd);
@@ -1944,7 +1880,7 @@ test_rc6(int gem_fd, unsigned int gt, unsigned int num_gt, unsigned int flags)
 			continue;
 
 		if (gt_ == gt) {
-			igt_assert(test_idx == -1);
+			igt_assert_eq(test_idx, -1);
 			test_idx = pmus;
 		}
 
@@ -1954,7 +1890,7 @@ test_rc6(int gem_fd, unsigned int gt, unsigned int num_gt, unsigned int flags)
 		igt_skip_on(fd[pmus] < 0 && errno == ENODEV);
 		pmus++;
 	}
-	igt_assert(test_idx >= 0);
+	igt_assert_lte(0, test_idx);
 
 	if (flags & TEST_RUNTIME_PM) {
 		drmModeRes *res;
@@ -2045,7 +1981,7 @@ test_rc6(int gem_fd, unsigned int gt, unsigned int num_gt, unsigned int flags)
 			continue;
 
 		fw[gt_] = open_forcewake_handle(gem_fd, gt_);
-		igt_assert(fw[gt_] >= 0);
+		igt_assert_lte(0, fw[gt_]);
 	}
 
 	usleep(1e3); /* wait for the rc6 cycle counter to stop ticking */
@@ -2270,7 +2206,8 @@ accuracy(int gem_fd, const intel_ctx_t *ctx,
 				 100 * expected, target_busy_pct,
 				 avg, sqrt(var / n));
 
-			write(link[1], &expected, sizeof(expected));
+			igt_assert_eq(write(link[1], &expected, sizeof(expected)),
+				      sizeof(expected));
 		}
 
 		igt_spin_free(gem_fd, spin);
@@ -2280,12 +2217,14 @@ accuracy(int gem_fd, const intel_ctx_t *ctx,
 	fd = open_pmu(gem_fd, I915_PMU_ENGINE_BUSY(e->class, e->instance));
 
 	/* Let the child run. */
-	read(link[0], &expected, sizeof(expected));
+	igt_assert_eq(read(link[0], &expected, sizeof(expected)),
+		      sizeof(expected));
 	assert_within(100.0 * expected, target_busy_pct, 5);
 
 	/* Collect engine busyness for an interesting part of child runtime. */
 	val[0] = __pmu_read_single(fd, &ts[0]);
-	read(link[0], &expected, sizeof(expected));
+	igt_assert_eq(read(link[0], &expected, sizeof(expected)),
+		      sizeof(expected));
 	val[1] = __pmu_read_single(fd, &ts[1]);
 	close(fd);
 
@@ -2454,12 +2393,69 @@ static void pmu_read(int i915)
 		for_each_if((e)->class == I915_ENGINE_CLASS_RENDER) \
 			igt_dynamic_f("%s", e->name)
 
+int fd = -1;
+uint32_t *stash_min, *stash_max, *stash_boost;
+
+static void save_sysfs_freq(int i915)
+{
+	int gt, num_gts, sysfs, tmp;
+	uint32_t rpn_freq, rp0_freq;
+
+	num_gts = igt_sysfs_get_num_gt(i915);
+
+	stash_min = (uint32_t *)malloc(sizeof(uint32_t) * num_gts);
+	stash_max = (uint32_t *)malloc(sizeof(uint32_t) * num_gts);
+	stash_boost = (uint32_t *)malloc(sizeof(uint32_t) * num_gts);
+
+	/* Save boost, min and max across GTs */
+	i915_for_each_gt(i915, tmp, gt) {
+		sysfs = igt_sysfs_gt_open(i915, gt);
+		igt_require(sysfs >= 0);
+
+		stash_min[gt] = igt_sysfs_get_u32(sysfs, "rps_min_freq_mhz");
+		stash_max[gt] = igt_sysfs_get_u32(sysfs, "rps_max_freq_mhz");
+		stash_boost[gt] = igt_sysfs_get_u32(sysfs, "rps_boost_freq_mhz");
+		igt_debug("GT: %d, min: %d, max: %d, boost:%d\n",
+			  gt, stash_min[gt], stash_max[gt], stash_boost[gt]);
+
+		rpn_freq = igt_sysfs_get_u32(sysfs, "rps_RPn_freq_mhz");
+		rp0_freq = igt_sysfs_get_u32(sysfs, "rps_RP0_freq_mhz");
+
+		/* Set pre-conditions, in case frequencies are in non-default state */
+		igt_require(__igt_sysfs_set_u32(sysfs, "rps_max_freq_mhz", rp0_freq));
+		igt_require(__igt_sysfs_set_u32(sysfs, "rps_boost_freq_mhz", rp0_freq));
+		igt_require(__igt_sysfs_set_u32(sysfs, "rps_min_freq_mhz", rpn_freq));
+
+		close(sysfs);
+	}
+}
+
+static void restore_sysfs_freq(int i915)
+{
+	int sysfs, gt, tmp;
+
+	/* Restore frequencies */
+	i915_for_each_gt(fd, tmp, gt) {
+		sysfs = igt_sysfs_gt_open(fd, gt);
+		igt_require(sysfs >= 0);
+
+		igt_require(__igt_sysfs_set_u32(sysfs, "rps_max_freq_mhz", stash_max[gt]));
+		igt_require(__igt_sysfs_set_u32(sysfs, "rps_min_freq_mhz", stash_min[gt]));
+		igt_require(__igt_sysfs_set_u32(sysfs, "rps_boost_freq_mhz", stash_boost[gt]));
+
+		close(sysfs);
+	}
+	free(stash_min);
+	free(stash_max);
+	free(stash_boost);
+}
+
 igt_main
 {
 	const struct intel_execution_engine2 *e;
 	unsigned int num_engines = 0;
 	const intel_ctx_t *ctx = NULL;
-	int gt, tmp, fd = -1;
+	int gt, tmp;
 	int num_gt = 0;
 
 	/**
@@ -2664,12 +2660,15 @@ igt_main
 	 * Test GPU frequency.
 	 */
 	igt_subtest_with_dynamic("frequency") {
+		save_sysfs_freq(fd);
+
 		i915_for_each_gt(fd, tmp, gt) {
 			igt_dynamic_f("gt%u", gt)
 				test_frequency(fd, gt);
 			igt_dynamic_f("idle-gt%u", gt)
 				test_frequency_idle(fd, gt);
 		}
+		restore_sysfs_freq(fd);
 	}
 
 	/**
