@@ -26,19 +26,60 @@
  * TEST: kms color
  * Category: Display
  * Description: Test Color Features at Pipe level
+ * Driver requirement: i915, xe
+ * Functionality: colorspace
+ * Mega feature: Color Management
+ * Test category: functionality test
  */
-#include "kms_color_helper.h"
 
-IGT_TEST_DESCRIPTION("Test Color Features at Pipe level");
+#include "kms_color_helper.h"
 
 /**
  * SUBTEST: degamma
  * Description: Verify that degamma LUT transformation works correctly
- * Driver requirement: i915, xe
- * Functionality: colorspace
- * Mega feature: Color mgmt
- * Test category: functionality test
+ *
+ * SUBTEST: gamma
+ * Description: Verify that gamma LUT transformation works correctly
+ *
+ * SUBTEST: legacy-gamma
+ * Description: Verify that legacy gamma LUT transformation works correctly
+ *
+ * SUBTEST: legacy-gamma-reset
+ * Description: Verify that setting the legacy gamma LUT resets the gamma LUT
+ *              set through GAMMA_LUT property
+ *
+ * SUBTEST: ctm-%s
+ * Description: Check the color transformation %arg[1]
+ *
+ * arg[1]:
+ *
+ * @0-25:           for 0.25 transparency
+ * @0-50:           for 0.50 transparency
+ * @0-75:           for 0.75 transparency
+ * @blue-to-red:    from blue to red
+ * @green-to-red:   from green to red
+ * @max:            for maximum transparency
+ * @negative:       for negative transparency
+ * @red-to-blue:    from red to blue
+ * @signed:         for correct signed handling
  */
+
+/**
+ * SUBTEST: deep-color
+ * Description: Verify that deep color works correctly
+ *
+ * SUBTEST: invalid-%s-sizes
+ * Description: Negative check for %arg[1] sizes
+ *
+ * arg[1]:
+ *
+ * @ctm-matrix:         Color transformation matrix
+ * @degamma-lut:        Degamma LUT
+ * @gamma-lut:          Gamma LUT
+ */
+
+IGT_TEST_DESCRIPTION("Test Color Features at Pipe level");
+
 static bool test_pipe_degamma(data_t *data,
 			      igt_plane_t *primary)
 {
@@ -127,15 +168,6 @@ static bool test_pipe_degamma(data_t *data,
 	return ret;
 }
 
-/**
- * SUBTEST: gamma
- * Description: Verify that gamma LUT transformation works correctly
- * Driver requirement: i915, xe
- * Functionality: colorspace
- * Mega feature: Color mgmt
- * Test category: functionality test
- */
-
 /*
  * Draw 3 gradient rectangles in red, green and blue, with a maxed out gamma
  * LUT and verify we have the same CRC as drawing solid color rectangles.
@@ -224,15 +256,6 @@ static bool test_pipe_gamma(data_t *data,
 	return ret;
 }
 
-/**
- * SUBTEST: legacy-gamma
- * Description: Verify that legacy gamma LUT transformation works correctly
- * Driver requirement: i915, xe
- * Functionality: colorspace
- * Mega feature: Color mgmt
- * Test category: functionality test
- */
-
 /*
  * Draw 3 gradient rectangles in red, green and blue, with a maxed out legacy
  * gamma LUT and verify we have the same CRC as drawing solid color rectangles
@@ -345,16 +368,6 @@ static bool test_pipe_legacy_gamma(data_t *data,
 	return ret;
 }
 
-/**
- * SUBTEST: legacy-gamma-reset
- * Description: Verify that setting the legacy gamma LUT resets the gamma LUT
- *              set through GAMMA_LUT property
- * Driver requirement: i915, xe
- * Functionality: colorspace
- * Mega feature: Color mgmt
- * Test category: functionality test
- */
-
 /*
  * Verify that setting the legacy gamma LUT resets the gamma LUT set
  * through the GAMMA_LUT property.
@@ -489,27 +502,6 @@ end:
 	return ret;
 }
 
-/**
- * SUBTEST: ctm-%s
- * Description: Check the color transformation %arg[1]
- * Driver requirement: i915, xe
- * Functionality: colorspace
- * Mega feature: Color mgmt
- * Test category: functionality test
- *
- * arg[1]:
- *
- * @0-25:           for 0.25 transparency
- * @0-50:           for 0.50 transparency
- * @0-75:           for 0.75 transparency
- * @blue-to-red:    from blue to red
- * @green-to-red:   from green to red
- * @max:            for maximum transparency
- * @negative:       for negative transparency
- * @red-to-blue:    from red to blue
- * @signed:         for correct signed handling
- */
-
 /*
  * Draw 3 rectangles using before colors with the ctm matrix apply and verify
  * the CRC is equal to using after colors with an identify ctm matrix.
@@ -798,8 +790,8 @@ run_gamma_degamma_tests_for_pipe(data_t *data, enum pipe p,
 	if (!pipe_output_combo_valid(data, p))
 		goto out;
 
-	igt_assert(test_t(data, data->primary));
-
+	igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(p), igt_output_name(data->output))
+		igt_assert(test_t(data, data->primary));
 out:
 	test_cleanup(data);
 }
@@ -840,43 +832,37 @@ run_ctm_tests_for_pipe(data_t *data, enum pipe p,
 	if (!iter)
 		iter = 1;
 
-	/*
-	 * We tests a few values around the expected result because
-	 * it depends on the hardware we're dealing with, we can either
-	 * get clamped or rounded values and we also need to account
-	 * for odd number of items in the LUTs.
-	 */
-	for (i = 0; i < iter; i++) {
-		color_t expected_colors[3] = {
-			fb_colors[0],
-			fb_colors[1],
-			fb_colors[2],
-		};
+	igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(p), igt_output_name(data->output)) {
+		/*
+		 * We tests a few values around the expected result because
+		 * it depends on the hardware we're dealing with, we can either
+		 * get clamped or rounded values and we also need to account
+		 * for odd number of items in the LUTs.
+		 */
+		for (i = 0; i < iter; i++) {
+			color_t expected_colors[3] = {
+				fb_colors[0],
+				fb_colors[1],
+				fb_colors[2],
+			};
 
-		transform_color(&expected_colors[0], ctm, delta * (i - (iter / 2)));
-		transform_color(&expected_colors[1], ctm, delta * (i - (iter / 2)));
-		transform_color(&expected_colors[2], ctm, delta * (i - (iter / 2)));
+			transform_color(&expected_colors[0], ctm, delta * (i - (iter / 2)));
+			transform_color(&expected_colors[1], ctm, delta * (i - (iter / 2)));
+			transform_color(&expected_colors[2], ctm, delta * (i - (iter / 2)));
 
-		if (test_pipe_ctm(data, data->primary, fb_colors,
-				  expected_colors, ctm)) {
-			success = true;
-			break;
+			if (test_pipe_ctm(data, data->primary, fb_colors,
+					  expected_colors, ctm)) {
+				success = true;
+				break;
+			}
 		}
+		igt_assert(success);
 	}
-	igt_assert(success);
 
 out:
 	test_cleanup(data);
 }
 
-/**
- * SUBTEST: deep-color
- * Description: Verify that deep color works correctly
- * Driver requirement: i915, xe
- * Functionality: colorspace
- * Mega feature: Color mgmt
- * Test category: functionality test
- */
 static void
 run_deep_color_tests_for_pipe(data_t *data, enum pipe p)
 {
@@ -907,11 +893,18 @@ run_deep_color_tests_for_pipe(data_t *data, enum pipe p)
 		uint64_t max_bpc = get_max_bpc(output);
 		bool ret;
 
-		if (!max_bpc)
+		if (!max_bpc) {
+			igt_info("Output %s: Doesn't support \"max bpc\" property.\n",
+				 igt_output_name(output));
 			continue;
+		}
 
-		if (!panel_supports_deep_color(data->drm_fd, output->name))
+		if (!panel_supports_deep_color(data->drm_fd, output->name)) {
+			igt_info("Output %s: Doesn't support deep-color.\n",
+				 igt_output_name(output));
 			continue;
+		}
+
 		/*
 		 * In intel driver, for MST streams pipe_bpp is
 		 * restricted to 8bpc. So, deep-color >= 10bpc
@@ -919,16 +912,22 @@ run_deep_color_tests_for_pipe(data_t *data, enum pipe p)
 		 * supports 10bpc. Once KMD FIXME, is resolved
 		 * this MST constraint can be removed.
 		 */
-		if (is_intel_device(data->drm_fd) && igt_check_output_is_dp_mst(output))
+		if (is_intel_device(data->drm_fd) && igt_check_output_is_dp_mst(output)) {
+			igt_info("Output %s: DP-MST doesn't support deep-color on Intel hardware.\n",
+				 igt_output_name(output));
 			continue;
+		}
 
 		igt_display_reset(&data->display);
 		igt_output_set_prop_value(output, IGT_CONNECTOR_MAX_BPC, 10);
 		igt_output_set_pipe(output, p);
 
 		if (is_intel_device(data->drm_fd) &&
-		    !igt_max_bpc_constraint(&data->display, p, output, 10))
+		    !igt_max_bpc_constraint(&data->display, p, output, 10)) {
+			igt_info("Output %s: Doesn't support 10-bpc.\n",
+				 igt_output_name(output));
 			continue;
+		}
 
 		data->color_depth = 10;
 		data->drm_format = DRM_FORMAT_XRGB2101010;
@@ -978,20 +977,6 @@ run_deep_color_tests_for_pipe(data_t *data, enum pipe p)
 	test_cleanup(data);
 }
 
-/**
- * SUBTEST: invalid-%s-sizes
- * Description: Negative check for %arg[1] sizes
- * Driver requirement: i915, xe
- * Functionality: colorspace
- * Mega feature: Color mgmt
- * Test category: functionality test
- *
- * arg[1]:
- *
- * @ctm-matrix:         Color transformation matrix
- * @degamma-lut:        Degamma LUT
- * @gamma-lut:          Gamma LUT
- */
 static void
 run_invalid_tests_for_pipe(data_t *data)
 {
@@ -1159,10 +1144,8 @@ run_tests_for_pipe(data_t *data)
 		igt_describe_f("%s", gamma_degamma_tests[i].desc);
 		igt_subtest_with_dynamic_f("%s", gamma_degamma_tests[i].name) {
 			for_each_pipe(&data->display, pipe) {
-				igt_dynamic_f("pipe-%s", kmstest_pipe_name(pipe)) {
-					run_gamma_degamma_tests_for_pipe(data, pipe,
-									 gamma_degamma_tests[i].test_t);
-				}
+				run_gamma_degamma_tests_for_pipe(data, pipe,
+								 gamma_degamma_tests[i].test_t);
 			}
 		}
 	}
@@ -1171,12 +1154,10 @@ run_tests_for_pipe(data_t *data)
 		igt_describe_f("%s", ctm_tests[i].desc);
 		igt_subtest_with_dynamic_f("%s", ctm_tests[i].name) {
 			for_each_pipe(&data->display, pipe) {
-				igt_dynamic_f("pipe-%s", kmstest_pipe_name(pipe)) {
-					run_ctm_tests_for_pipe(data, pipe,
-							       ctm_tests[i].fb_colors,
-							       ctm_tests[i].ctm,
-							       ctm_tests[i].iter);
-				}
+				run_ctm_tests_for_pipe(data, pipe,
+						       ctm_tests[i].fb_colors,
+						       ctm_tests[i].ctm,
+						       ctm_tests[i].iter);
 			}
 		}
 	}
