@@ -21,6 +21,17 @@
  * IN THE SOFTWARE.
  */
 
+/**
+ * TEST: kms rmfb
+ * Category: Display
+ * Description: This tests rmfb and close-fd behavior. In these cases the
+ *              framebuffers should be removed from the crtc.
+ * Driver requirement: i915, xe
+ * Functionality: kms_gem_interop
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
+
 #include "igt.h"
 #include "drmtest.h"
 #include <errno.h>
@@ -28,27 +39,16 @@
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
+
 /**
- * TEST: kms rmfb
- * Category: Display
- * Description: This tests rmfb and close-fd behavior. In these casesthe
- *              framebuffers should be removed from the crtc.
- *
  * SUBTEST: close-fd
- * Description: RMFB is supposed to free the framebuffers from any and all planes
- *              so test this and make sure it works with fd close and reopen.
- * Driver requirement: i915, xe
- * Functionality: kms_gem_interop
- * Mega feature: General Display Features
- * Test category: functionality test
+ * Description: Kernel driver is supposed to free the framebuffers from any and all planes
+ *		when the fd is closed. Ensure that is the case by closing and re-opening
+ *		it.
  *
  * SUBTEST: rmfb-ioctl
- * Description: RMFB is supposed to free the framebuffers from any and all planes
- *              so test this and make sure it works.
- * Driver requirement: i915, xe
- * Functionality: kms_gem_interop
- * Mega feature: General Display Features
- * Test category: functionality test
+ * Description: Kernel driver is supposed to free the framebuffers from any and all planes
+ *              when DRM_IOCTL_MODE_RMFB ioctl is called. Ensure that is the case.
  */
 
 #ifndef DRM_CAP_CURSOR_WIDTH
@@ -58,7 +58,7 @@
 #define DRM_CAP_CURSOR_HEIGHT 0x9
 #endif
 
-IGT_TEST_DESCRIPTION("This tests rmfb and close-fd behavior. In these cases"
+IGT_TEST_DESCRIPTION("This tests rmfb and close-fd behavior. In these cases "
 		     "the framebuffers should be removed from the crtc.");
 
 struct rmfb_data {
@@ -183,7 +183,7 @@ run_rmfb_test(struct rmfb_data *data, bool reopen)
 		igt_display_reset(display);
 
 		igt_output_set_pipe(output, pipe);
-		if (!i915_pipe_output_combo_valid(display))
+		if (!intel_pipe_output_combo_valid(display))
 			continue;
 
 		igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe),
@@ -199,11 +199,11 @@ igt_main
 		const char *name;
 		const char *description;
 	} tests[] = {
-		{ false, "rmfb-ioctl", "RMFB is supposed to free the framebuffers from any and all "
-				       "planes so test this and make sure it works."},
-		{ true, "close-fd", "RMFB is supposed to free the framebuffers from any and all "
-				    "planes so test this and make sure it works with fd close "
-				    "and reopen."},
+		{ false, "rmfb-ioctl", "Kernel driver is supposed to free the framebuffers from any and all planes "
+				       "when DRM_IOCTL_MODE_RMFB ioctl is called. Ensure that is the case." },
+		{ true, "close-fd", "Kernel driver is supposed to free the framebuffers from any and all planes "
+				    "when the fd is closed. Ensure that is the case by closing and re-opening "
+				    "it" },
 	};
 	struct rmfb_data data = {};
 	int i;
