@@ -21,6 +21,16 @@
  * IN THE SOFTWARE.
  */
 
+/**
+ * TEST: kms universal plane
+ * Category: Display
+ * Description: Check pageflip & modeset on universal plane
+ * Driver requirement: i915, xe
+ * Functionality: plane
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
+
 #include "igt.h"
 #include <errno.h>
 #include <stdbool.h>
@@ -29,65 +39,22 @@
 #include <fcntl.h>
 
 /**
- * TEST: kms universal plane
- * Category: Display
- * Description: Check pageflip & modeset on universal plane
- *
- * SUBTEST: cursor-fb-leak-pipe-%s
- * Description: Check for cursor leaks after performing cursor operations on %arg[1]
- * Driver requirement: i915, xe
+ * SUBTEST: cursor-fb-leak
+ * Description: Check for cursor leaks after performing cursor operations
  * Functionality: cursor, plane
- * Mega feature: General Display Features
- * Test category: functionality test
  *
- * SUBTEST: disable-primary-vs-flip-pipe-%s
+ * SUBTEST: disable-primary-vs-flip
  * Description: Check pageflips while primary plane is disabled before IOCTL or
- *              between IOCTL and pageflip execution on %arg[1]
- * Driver requirement: i915, xe
- * Functionality: plane
- * Mega feature: General Display Features
- * Test category: functionality test
+ *              between IOCTL and pageflip execution
  *
- * SUBTEST: universal-plane-pageflip-windowed-pipe-%s
- * Description: Check if pageflip succeeds in windowed setting on %arg[1]
- * Driver requirement: i915, xe
- * Functionality: plane
- * Mega feature: General Display Features
- * Test category: functionality test
+ * SUBTEST: universal-plane-pageflip-windowed
+ * Description: Check if pageflip succeeds in windowed setting
+ *
+ * SUBTEST: universal-plane-%s
+ * Description: Check %arg[1]
  *
  * arg[1]:
  *
- * @A:    pipe A
- * @B:    pipe B
- * @C:    pipe C
- * @D:    pipe D
- * @E:    pipe E
- * @F:    pipe F
- * @G:    pipe G
- * @H:    pipe H
- */
-
-/**
- * SUBTEST: universal-plane-pipe-%s-%s
- * Description: Check %arg[2] on %arg[1]
- * Driver requirement: i915, xe
- * Functionality: plane
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * arg[1]:
- *
- * @A:    pipe A
- * @B:    pipe B
- * @C:    pipe C
- * @D:    pipe D
- * @E:    pipe E
- * @F:    pipe F
- * @G:    pipe G
- * @H:    pipe H
- *
- * arg[2]:
- *
  * @functional: the switching between different primary plane fbs with CRTC off
  * @sanity:     for scale-up or scale-down using universal plane API without
  *              covering CRTC
@@ -172,7 +139,7 @@ functional_test_fini(functional_test_t *test, igt_output_t *output)
 	igt_remove_fb(test->data->drm_fd, &test->red_fb);
 	igt_remove_fb(test->data->drm_fd, &test->yellow_fb);
 
-	igt_output_set_pipe(output, PIPE_ANY);
+	igt_output_set_pipe(output, PIPE_NONE);
 	igt_display_commit2(&test->data->display, COMMIT_LEGACY);
 }
 
@@ -200,11 +167,6 @@ functional_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	int num_primary = 0, num_cursor = 0;
 	int i;
 
-	igt_require_pipe(display, pipe);
-
-	igt_info("Testing connector %s using pipe %s\n", igt_output_name(output),
-		 kmstest_pipe_name(pipe));
-
 	functional_test_init(&test, output, pipe);
 
 	/*
@@ -424,7 +386,7 @@ sanity_test_fini(sanity_test_t *test, igt_output_t *output)
 	igt_remove_fb(test->data->drm_fd, &test->undersized_fb);
 	igt_remove_fb(test->data->drm_fd, &test->blue_fb);
 
-	igt_output_set_pipe(output, PIPE_ANY);
+	igt_output_set_pipe(output, PIPE_NONE);
 	igt_display_commit2(&test->data->display, COMMIT_LEGACY);
 }
 
@@ -440,14 +402,9 @@ sanity_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	sanity_test_t test = { .data = data };
 	igt_plane_t *primary;
 	drmModeModeInfo *mode;
-	int i;
+	int i, ret;
 	int expect = 0;
 
-	igt_require_pipe(&data->display, pipe);
-
-	igt_info("Using (pipe %s + %s) to run the subtest.\n",
-		 kmstest_pipe_name(pipe), igt_output_name(output));
-
 	igt_output_set_pipe(output, pipe);
 	mode = igt_output_get_mode(output);
 
@@ -475,29 +432,43 @@ sanity_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 
 	igt_plane_set_position(primary, 0, 0);
 
-	/* Try to use universal plane API to scale down (should fail on pre-gen9) */
+	/* Try to use universal plane API to scale down */
 	if (is_intel_device(data->drm_fd))
 		expect = (data->display_ver < 9) ? -ERANGE : 0;
-	igt_assert(drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
-				   output->config.crtc->crtc_id,
-				   test.oversized_fb.fb_id, 0,
-				   0, 0,
-				   mode->hdisplay + 100,
-				   mode->vdisplay + 100,
-				   IGT_FIXED(0,0), IGT_FIXED(0,0),
-				   IGT_FIXED(mode->hdisplay,0),
-				   IGT_FIXED(mode->vdisplay,0)) == expect);
+	ret = drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
+			      output->config.crtc->crtc_id,
+			      test.oversized_fb.fb_id, 0,
+			      0, 0,
+			      mode->hdisplay + 100,
+			      mode->vdisplay + 100,
+			      IGT_FIXED(0,0), IGT_FIXED(0,0),
+			      IGT_FIXED(mode->hdisplay,0),
+			      IGT_FIXED(mode->vdisplay,0));
+	if (is_intel_device(data->drm_fd)) {
+		/* should fail on pre-gen9 */
+		igt_assert_eq(ret, (data->display_ver < 9) ? -ERANGE : 0);
+	} else {
+		/* Could succeed or fail with -ERANGE depending on hw limits: */
+		igt_assert((ret == 0) || (ret == -ERANGE));
+	}
 
-	/* Try to use universal plane API to scale up (should fail on pre-gen9) */
-	igt_assert(drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
-				   output->config.crtc->crtc_id,
-				   test.oversized_fb.fb_id, 0,
-				   0, 0,
-				   mode->hdisplay,
-				   mode->vdisplay,
-				   IGT_FIXED(0,0), IGT_FIXED(0,0),
-				   IGT_FIXED(mode->hdisplay - 100,0),
-				   IGT_FIXED(mode->vdisplay - 100,0)) == expect);
+	/* Try to use universal plane API to scale up */
+	ret = drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
+			      output->config.crtc->crtc_id,
+			      test.oversized_fb.fb_id, 0,
+			      0, 0,
+			      mode->hdisplay,
+			      mode->vdisplay,
+			      IGT_FIXED(0,0), IGT_FIXED(0,0),
+			      IGT_FIXED(mode->hdisplay - 100,0),
+			      IGT_FIXED(mode->vdisplay - 100,0));
+	if (is_intel_device(data->drm_fd)) {
+		/* should fail on pre-gen9 */
+		igt_assert_eq(ret, (data->display_ver < 9) ? -ERANGE : 0);
+	} else {
+		/* Could succeed or fail with -ERANGE depending on hw limits: */
+		igt_assert((ret == 0) || (ret == -ERANGE));
+	}
 
 	/* Find other crtcs and try to program our primary plane on them */
 	for (i = 0; i < test.moderes->count_crtcs; i++)
@@ -545,7 +516,7 @@ pageflip_test_fini(pageflip_test_t *test, igt_output_t *output)
 	igt_remove_fb(test->data->drm_fd, &test->red_fb);
 	igt_remove_fb(test->data->drm_fd, &test->blue_fb);
 
-	igt_output_set_pipe(output, PIPE_ANY);
+	igt_output_set_pipe(output, PIPE_NONE);
 	igt_display_commit2(&test->data->display, COMMIT_LEGACY);
 }
 
@@ -560,11 +531,6 @@ pageflip_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	fd_set fds;
 	int ret = 0;
 
-	igt_require_pipe(&data->display, pipe);
-
-	igt_info("Using (pipe %s + %s) to run the subtest.\n",
-		 kmstest_pipe_name(pipe), igt_output_name(output));
-
 	igt_output_set_pipe(output, pipe);
 
 	pageflip_test_init(&test, output, pipe);
@@ -637,7 +603,7 @@ cursor_leak_test_fini(data_t *data,
 	for (i = 0; i < 10; i++)
 		igt_remove_fb(data->drm_fd, &curs[i]);
 
-	igt_output_set_pipe(output, PIPE_ANY);
+	igt_output_set_pipe(output, PIPE_NONE);
 }
 
 static int
@@ -676,13 +642,9 @@ cursor_leak_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	int r, g, b;
 	int count1, count2;
 
-	igt_require_pipe(display, pipe);
 	igt_require(display->has_cursor_plane);
 	igt_require_intel(data->drm_fd);
 
-	igt_info("Using (pipe %s + %s) to run the subtest.\n",
-		 kmstest_pipe_name(pipe), igt_output_name(output));
-
 	igt_output_set_pipe(output, pipe);
 	mode = igt_output_get_mode(output);
 
@@ -717,7 +679,6 @@ cursor_leak_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 		igt_skip("Primary and/or cursor are unavailable\n");
 	}
 
-
 	igt_plane_set_fb(primary, &background_fb);
 	igt_display_commit2(display, COMMIT_LEGACY);
 
@@ -791,7 +752,7 @@ gen9_test_fini(gen9_test_t *test, igt_output_t *output)
 	igt_remove_fb(test->data->drm_fd, &test->smallred_fb);
 	igt_remove_fb(test->data->drm_fd, &test->smallblue_fb);
 
-	igt_output_set_pipe(output, PIPE_ANY);
+	igt_output_set_pipe(output, PIPE_NONE);
 	igt_display_commit2(&test->data->display, COMMIT_LEGACY);
 }
 
@@ -807,12 +768,6 @@ pageflip_win_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 
 	int ret = 0;
 
-	igt_skip_on(is_intel_device(data->drm_fd) && data->display_ver < 9);
-	igt_require_pipe(&data->display, pipe);
-
-	igt_info("Using (pipe %s + %s) to run the subtest.\n",
-		 kmstest_pipe_name(pipe), igt_output_name(output));
-
 	igt_output_set_pipe(output, pipe);
 
 	gen9_test_init(&test, output, pipe);
@@ -857,7 +812,7 @@ pipe_output_combo_valid(igt_display_t *display,
 	igt_display_reset(display);
 
 	igt_output_set_pipe(output, pipe);
-	if (!i915_pipe_output_combo_valid(display))
+	if (!intel_pipe_output_combo_valid(display))
 		ret = false;
 	igt_output_set_pipe(output, PIPE_NONE);
 
@@ -865,103 +820,72 @@ pipe_output_combo_valid(igt_display_t *display,
 }
 
 static void
-run_tests_for_pipe(data_t *data, enum pipe pipe)
+run_tests(data_t *data)
 {
 	igt_output_t *output;
-
-	igt_fixture {
-		int valid_tests = 0;
-
-		igt_require_pipe(&data->display, pipe);
-
-		for_each_valid_output_on_pipe(&data->display, pipe, output)
-			valid_tests++;
-
-		igt_require_f(valid_tests, "no valid crtc/connector combinations found\n");
-	}
+	enum pipe pipe;
 
 	igt_describe("Check the switching between different primary plane fbs with CRTC off");
-	igt_subtest_f("universal-plane-pipe-%s-functional",
-		      kmstest_pipe_name(pipe)) {
-		bool found = false;
-
-		for_each_valid_output_on_pipe(&data->display, pipe, output) {
+	igt_subtest_with_dynamic("universal-plane-functional") {
+		for_each_pipe_with_single_output(&data->display, pipe, output) {
 			if (!pipe_output_combo_valid(&data->display, pipe, output))
 				continue;
 
-			found = true;
-			functional_test_pipe(data, pipe, output);
+			igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output))
+				functional_test_pipe(data, pipe, output);
 		}
-		igt_require_f(found, "No valid pipe/output combo found.\n");
 	}
 
 	igt_describe("Test for scale-up or scale-down using universal plane API without covering CRTC");
-	igt_subtest_f("universal-plane-pipe-%s-sanity",
-		      kmstest_pipe_name(pipe)) {
-		bool found = false;
-
-		for_each_valid_output_on_pipe(&data->display, pipe, output) {
+	igt_subtest_with_dynamic("universal-plane-sanity") {
+		for_each_pipe_with_single_output(&data->display, pipe, output) {
 			if (!pipe_output_combo_valid(&data->display, pipe, output))
 				continue;
 
-			found = true;
-			sanity_test_pipe(data, pipe, output);
+			igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output))
+				sanity_test_pipe(data, pipe, output);
 		}
-		igt_require_f(found, "No valid pipe/output combo found.\n");
 	}
 
 	igt_describe("Check pageflips while primary plane is disabled before IOCTL or between IOCTL"
-			" and pageflip execution");
-	igt_subtest_f("disable-primary-vs-flip-pipe-%s",
-		      kmstest_pipe_name(pipe)) {
-		bool found = false;
-
-		for_each_valid_output_on_pipe(&data->display, pipe, output) {
+		     " and pageflip execution");
+	igt_subtest_with_dynamic("disable-primary-vs-flip") {
+		for_each_pipe_with_single_output(&data->display, pipe, output) {
 			if (!pipe_output_combo_valid(&data->display, pipe, output))
 				continue;
 
-			found = true;
-			pageflip_test_pipe(data, pipe, output);
+			igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output))
+				pageflip_test_pipe(data, pipe, output);
 		}
-		igt_require_f(found, "No valid pipe/output combo found.\n");
 	}
 
 	igt_describe("Check for cursor leaks after performing cursor operations");
-	igt_subtest_f("cursor-fb-leak-pipe-%s",
-		      kmstest_pipe_name(pipe)) {
-		bool found = false;
-
-		for_each_valid_output_on_pipe(&data->display, pipe, output) {
+	igt_subtest_with_dynamic("cursor-fb-leak") {
+		for_each_pipe_with_single_output(&data->display, pipe, output) {
 			if (!pipe_output_combo_valid(&data->display, pipe, output))
 				continue;
 
-			found = true;
-			cursor_leak_test_pipe(data, pipe, output);
+			igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output))
+				cursor_leak_test_pipe(data, pipe, output);
 		}
-		igt_require_f(found, "No valid pipe/output combo found.\n");
 	}
 
 	igt_describe("Check if pageflip succeeds in windowed setting");
-	igt_subtest_f("universal-plane-pageflip-windowed-pipe-%s",
-		      kmstest_pipe_name(pipe)) {
-		bool found = false;
-
-		for_each_valid_output_on_pipe(&data->display, pipe, output) {
+	igt_subtest_with_dynamic("universal-plane-pageflip-windowed") {
+		igt_require(is_intel_device(data->drm_fd) && data->display_ver >= 9);
+		for_each_pipe_with_single_output(&data->display, pipe, output) {
 			if (!pipe_output_combo_valid(&data->display, pipe, output))
 				continue;
 
-			found = true;
-			pageflip_win_test_pipe(data, pipe, output);
+			igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output))
+				pageflip_win_test_pipe(data, pipe, output);
 		}
-		igt_require_f(found, "No valid pipe/output combo found.\n");
 	}
 }
 
-static data_t data;
-
 igt_main
 {
-	enum pipe pipe;
+	data_t data;
 
 	igt_fixture {
 		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
@@ -975,10 +899,7 @@ igt_main
 		igt_display_require_output(&data.display);
 	}
 
-	for_each_pipe_static(pipe) {
-		igt_subtest_group
-			run_tests_for_pipe(&data, pipe);
-	}
+	run_tests(&data);
 
 	igt_fixture {
 		igt_display_fini(&data.display);
