@@ -22,6 +22,19 @@
  *
  */
 
+/**
+ * TEST: kms writeback
+ * Category: Display
+ * Description: This test validates the expected behavior of the writeback
+ *              connectors feature by checking if the target device support
+ *              writeback; it validates bad and good combination, check color
+ *              format, and check the output result by using CRC.
+ * Driver requirement: i915, xe
+ * Functionality: kms_core
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
+
 #include <errno.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -32,43 +45,27 @@
 #include "igt_core.h"
 #include "igt_fb.h"
 #include "sw_sync.h"
+
 /**
- * TEST: kms writeback
- * Category: Display
- * Description: This test validates the expected behavior of the writeback
- *              connectors feature by checking if the target device support
- *              writeback; it validates bad and good combination, check color
- *              format, and check the output result by using CRC.
+ * SUBTEST: writeback-check-output-XRGB2101010
+ * Description: Check XRGB2101010 writeback output with CRC validation
  *
  * SUBTEST: writeback-check-output
  * Description: Check writeback output with CRC validation
- * Driver requirement: i915, xe
- * Functionality: kms_core
- * Mega feature: General Display Features
- * Test category: functionality test
+ *
+ * SUBTEST: writeback-fb-id-XRGB2101010
+ * Description: Validate WRITEBACK_FB_ID with valid and invalid options
  *
  * SUBTEST: writeback-fb-id
  * Description: Validate WRITEBACK_FB_ID with valid and invalid options
- * Driver requirement: i915, xe
- * Functionality: kms_core
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * SUBTEST: writeback-invalid-parameters
  * Description: Writeback has a couple of parameters linked together(output
  *              framebuffer and fence); this test goes throughthe combination
  *              of possible bad options
- * Driver requirement: i915, xe
- * Functionality: kms_core
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * SUBTEST: writeback-pixel-formats
  * Description: Check the writeback format
- * Driver requirement: i915, xe
- * Functionality: kms_core
- * Mega feature: General Display Features
- * Test category: functionality test
  */
 
 IGT_TEST_DESCRIPTION(
@@ -85,12 +82,18 @@ typedef struct {
 	bool dump_check;
 	bool wb_fmt;
 	uint32_t format;
+	uint64_t supported_colors;
 	int mode_index;
 	drmModeModeInfo user_mode;
 } data_t;
 
 static data_t data;
 
+enum {
+	XRGB8888 = 1 << 0,
+	XRGB2101010 = 1 << 1,
+};
+
 static drmModePropertyBlobRes *get_writeback_formats_blob(igt_output_t *output)
 {
 	drmModePropertyBlobRes *blob = NULL;
@@ -115,34 +118,45 @@ static bool check_writeback_config(igt_display_t *display, igt_output_t *output,
 {
 	igt_fb_t input_fb, output_fb;
 	igt_plane_t *plane;
-	uint32_t writeback_format = DRM_FORMAT_XRGB8888;
 	uint64_t modifier = DRM_FORMAT_MOD_LINEAR;
 	int width, height, ret;
+	uint16_t i;
+
+	const uint32_t fourcc[] = {
+		DRM_FORMAT_XRGB8888,
+		DRM_FORMAT_XRGB2101010,
+	};
 
 	igt_output_override_mode(output, &override_mode);
 
 	width = override_mode.hdisplay;
 	height = override_mode.vdisplay;
 
-	ret = igt_create_fb(display->drm_fd, width, height,
-			    DRM_FORMAT_XRGB8888, modifier, &input_fb);
-	igt_assert(ret >= 0);
+	for (i = 0; i < sizeof(fourcc) / sizeof(uint32_t); i++) {
 
-	ret = igt_create_fb(display->drm_fd, width, height,
-			    writeback_format, modifier, &output_fb);
-	igt_assert(ret >= 0);
+		ret = igt_create_fb(display->drm_fd, width, height,
+				    fourcc[i], modifier, &input_fb);
+		igt_assert_lte(0, ret);
 
-	plane = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
-	igt_plane_set_fb(plane, &input_fb);
-	igt_output_set_writeback_fb(output, &output_fb);
+		ret = igt_create_fb(display->drm_fd, width, height,
+				    fourcc[i], modifier, &output_fb);
+		igt_assert_lte(0, ret);
 
-	ret = igt_display_try_commit_atomic(display, DRM_MODE_ATOMIC_TEST_ONLY |
-					    DRM_MODE_ATOMIC_ALLOW_MODESET, NULL);
-	igt_plane_set_fb(plane, NULL);
-	igt_remove_fb(display->drm_fd, &input_fb);
-	igt_remove_fb(display->drm_fd, &output_fb);
+		plane = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
+		igt_plane_set_fb(plane, &input_fb);
+		igt_output_set_writeback_fb(output, &output_fb);
 
-	return !ret;
+		ret = igt_display_try_commit_atomic(display, DRM_MODE_ATOMIC_TEST_ONLY |
+						    DRM_MODE_ATOMIC_ALLOW_MODESET, NULL);
+		igt_plane_set_fb(plane, NULL);
+		igt_remove_fb(display->drm_fd, &input_fb);
+		igt_remove_fb(display->drm_fd, &output_fb);
+
+		if (!ret)
+			data.supported_colors |= 1 << i;
+	}
+
+	return data.supported_colors;
 }
 
 static igt_output_t *kms_writeback_get_output(igt_display_t *display)
@@ -279,7 +293,7 @@ static void test_invalid_parameters(igt_output_t *output, igt_fb_t *valid_fb, ig
 		ret = do_writeback_test(output, invalid_tests[i].fb_id,
 					invalid_tests[i].out_fence_ptr,
 					invalid_tests[i].ptr_valid);
-		igt_assert(ret != 0);
+		igt_assert_neq(ret, 0);
 	}
 }
 
@@ -294,11 +308,11 @@ static void writeback_fb_id(igt_output_t *output, igt_fb_t *valid_fb, igt_fb_t *
 
 	/* Zero WRITEBACK_FB_ID */
 	ret = do_writeback_test(output, 0, NULL, false);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/* Valid output buffer */
 	ret = do_writeback_test(output, valid_fb->fb_id, NULL, false);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 }
 
 static void fill_fb(igt_fb_t *fb, uint32_t pixel)
@@ -306,7 +320,7 @@ static void fill_fb(igt_fb_t *fb, uint32_t pixel)
 	uint32_t *ptr;
 	int64_t pixel_count, i;
 
-	igt_assert(fb->drm_format == DRM_FORMAT_XRGB8888);
+	igt_assert(fb->drm_format == DRM_FORMAT_XRGB8888 || fb->drm_format == DRM_FORMAT_XRGB2101010);
 
 	ptr = igt_fb_map_buffer(fb->fd, fb);
 	igt_assert(ptr);
@@ -331,14 +345,21 @@ static void get_and_wait_out_fence(igt_output_t *output)
 }
 
 static void writeback_sequence(igt_output_t *output, igt_plane_t *plane,
-				igt_fb_t *in_fb, igt_fb_t *out_fbs[], int n_commits)
+			       igt_fb_t *in_fb, igt_fb_t *out_fbs[],
+			       int n_commits, uint32_t fourcc_color)
 {
 	int i = 0;
-	uint32_t in_fb_colors[2] = { 0x42ff0000, 0x4200ff00 };
+	uint32_t *in_fb_colors;
+	uint32_t in_fb_colors_8bits[2] = { 0x42ff0000, 0x4200ff00 };
+	uint32_t in_fb_colors_10bits[2] = { 0x3ff00000, 0x000ffc00 };
 	uint32_t clear_color = 0xffffffff;
-
 	igt_crc_t cleared_crc, out_expected;
 
+	if (fourcc_color == DRM_FORMAT_XRGB2101010)
+		in_fb_colors = in_fb_colors_10bits;
+	else
+		in_fb_colors = in_fb_colors_8bits;
+
 	for (i = 0; i < n_commits; i++) {
 		/* Change the input color each time */
 		fill_fb(in_fb, in_fb_colors[i % 2]);
@@ -386,32 +407,33 @@ static void writeback_sequence(igt_output_t *output, igt_plane_t *plane,
 }
 
 static void writeback_check_output(igt_output_t *output, igt_plane_t *plane,
-				   igt_fb_t *input_fb, igt_fb_t *output_fb)
+				   igt_fb_t *input_fb, igt_fb_t *output_fb,
+				   uint32_t fourcc_color)
 {
 	igt_fb_t *out_fbs[2] = { 0 };
 	igt_fb_t second_out_fb;
 	unsigned int fb_id;
 
 	/* One commit, with a writeback. */
-	writeback_sequence(output, plane, input_fb, &output_fb, 1);
+	writeback_sequence(output, plane, input_fb, &output_fb, 1, fourcc_color);
 
 	/* Two commits, the second with no writeback */
 	out_fbs[0] = output_fb;
-	writeback_sequence(output, plane, input_fb, out_fbs, 2);
+	writeback_sequence(output, plane, input_fb, out_fbs, 2, fourcc_color);
 
 	/* Two commits, both with writeback */
 	out_fbs[1] = output_fb;
-	writeback_sequence(output, plane, input_fb, out_fbs, 2);
+	writeback_sequence(output, plane, input_fb, out_fbs, 2, fourcc_color);
 
 	fb_id = igt_create_fb(output_fb->fd, output_fb->width, output_fb->height,
-			      DRM_FORMAT_XRGB8888,
+			      fourcc_color,
 			      igt_fb_mod_to_tiling(0),
 			      &second_out_fb);
 	igt_require(fb_id > 0);
 
 	/* Two commits, with different writeback buffers */
 	out_fbs[1] = &second_out_fb;
-	writeback_sequence(output, plane, input_fb, out_fbs, 2);
+	writeback_sequence(output, plane, input_fb, out_fbs, 2, fourcc_color);
 
 	igt_remove_fb(output_fb->fd, &second_out_fb);
 }
@@ -533,18 +555,25 @@ igt_main_args("b:c:f:dl", long_options, help_str, opt_handler, NULL)
 	igt_display_t display;
 	igt_output_t *output;
 	igt_plane_t *plane;
-	igt_fb_t input_fb;
+	igt_fb_t input_fb, input_fb_10bit;
 	drmModeModeInfo mode;
 	unsigned int fb_id;
+	int ret;
 
 	memset(&display, 0, sizeof(display));
 
 	igt_fixture {
 		display.drm_fd = drm_open_driver_master(DRIVER_ANY);
-		igt_display_require(&display, display.drm_fd);
 
 		kmstest_set_vt_graphics_mode();
 
+		if (drmSetClientCap(display.drm_fd, DRM_CLIENT_CAP_ATOMIC, 1) == 0)
+			display.is_atomic = 1;
+
+		ret = drmSetClientCap(display.drm_fd, DRM_CLIENT_CAP_WRITEBACK_CONNECTORS, 1);
+
+		igt_require_f(!ret, "error setting DRM_CLIENT_CAP_WRITEBACK_CONNECTORS\n");
+
 		igt_display_require(&display, display.drm_fd);
 
 		igt_require(display.is_atomic);
@@ -566,6 +595,14 @@ igt_main_args("b:c:f:dl", long_options, help_str, opt_handler, NULL)
 				      DRM_FORMAT_MOD_LINEAR,
 				      &input_fb);
 		igt_assert(fb_id >= 0);
+
+		fb_id = igt_create_fb(display.drm_fd, mode.hdisplay,
+				      mode.vdisplay,
+				      DRM_FORMAT_XRGB2101010,
+				      DRM_FORMAT_MOD_LINEAR,
+				      &input_fb_10bit);
+		igt_assert(fb_id >= 0);
+
 		igt_plane_set_fb(plane, &input_fb);
 
 		if (data.list_modes)
@@ -625,6 +662,7 @@ igt_main_args("b:c:f:dl", long_options, help_str, opt_handler, NULL)
 		igt_fb_t output_fb;
 
 		igt_skip_on(data.dump_check || data.list_modes);
+		igt_skip_on_f(!(data.supported_colors & XRGB8888),"DRM_FORMAT_XRGB8888 is unsupported\n");
 		fb_id = igt_create_fb(display.drm_fd, mode.hdisplay, mode.vdisplay,
 				      DRM_FORMAT_XRGB8888,
 				      DRM_FORMAT_MOD_LINEAR,
@@ -636,18 +674,53 @@ igt_main_args("b:c:f:dl", long_options, help_str, opt_handler, NULL)
 		igt_remove_fb(display.drm_fd, &output_fb);
 	}
 
+	igt_describe("Validate XRGB2101010 WRITEBACK_FB_ID with valid and invalid options");
+	igt_subtest("writeback-fb-id-XRGB2101010") {
+		igt_fb_t output_fb;
+
+		igt_skip_on(data.dump_check || data.list_modes);
+		igt_skip_on_f(!(data.supported_colors & XRGB2101010), "DRM_FORMAT_XRGB2101010 is unsupported\n");
+		fb_id = igt_create_fb(display.drm_fd, mode.hdisplay, mode.vdisplay,
+				      DRM_FORMAT_XRGB2101010,
+				      DRM_FORMAT_MOD_LINEAR,
+				      &output_fb);
+		igt_require(fb_id > 0);
+
+		writeback_fb_id(output, &input_fb_10bit, &output_fb);
+
+		igt_remove_fb(display.drm_fd, &output_fb);
+	}
+
 	igt_describe("Check writeback output with CRC validation");
 	igt_subtest("writeback-check-output") {
 		igt_fb_t output_fb;
 
 		igt_skip_on(data.dump_check || data.list_modes);
+		igt_skip_on_f(!(data.supported_colors & XRGB8888),"DRM_FORMAT_XRGB8888 is unsupported\n");
 		fb_id = igt_create_fb(display.drm_fd, mode.hdisplay, mode.vdisplay,
 				      DRM_FORMAT_XRGB8888,
 				      igt_fb_mod_to_tiling(0),
 				      &output_fb);
 		igt_require(fb_id > 0);
 
-		writeback_check_output(output, plane, &input_fb, &output_fb);
+		writeback_check_output(output, plane, &input_fb, &output_fb, DRM_FORMAT_XRGB8888);
+
+		igt_remove_fb(display.drm_fd, &output_fb);
+	}
+
+	igt_describe("Check XRGB2101010 writeback output with CRC validation");
+	igt_subtest("writeback-check-output-XRGB2101010") {
+		igt_fb_t output_fb;
+
+		igt_skip_on(data.dump_check || data.list_modes);
+		igt_skip_on_f(!(data.supported_colors & XRGB2101010), "DRM_FORMAT_XRGB2101010 is unsupported\n");
+		fb_id = igt_create_fb(display.drm_fd, mode.hdisplay, mode.vdisplay,
+				      DRM_FORMAT_XRGB2101010,
+				      igt_fb_mod_to_tiling(0),
+				      &output_fb);
+		igt_require(fb_id > 0);
+
+		writeback_check_output(output, plane, &input_fb_10bit, &output_fb, DRM_FORMAT_XRGB2101010);
 
 		igt_remove_fb(display.drm_fd, &output_fb);
 	}
@@ -655,6 +728,7 @@ igt_main_args("b:c:f:dl", long_options, help_str, opt_handler, NULL)
 	igt_fixture {
 		detach_crtc(&display, output);
 		igt_remove_fb(display.drm_fd, &input_fb);
+		igt_remove_fb(display.drm_fd, &input_fb_10bit);
 		igt_display_fini(&display);
 		drm_close_driver(display.drm_fd);
 	}
