@@ -21,140 +21,87 @@
  * IN THE SOFTWARE.
  */
 
+#include <poll.h>
 #include <sys/ioctl.h>
-#include <sys/poll.h>
 #include <time.h>
 
 #include "i915/gem.h"
 #include "i915/gem_create.h"
 #include "igt.h"
 #include "igt_vgem.h"
-#include "intel_batchbuffer.h"	/* igt_blitter_src_copy() */
+#include "intel_batchbuffer.h"	/* igt_blitter_copy() */
 /**
  * TEST: prime vgem
  * Description: Basic check of polling for prime/vgem fences.
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: DRM
+ * Functionality: mock device
+ * Feature: prime
+ * Test category: GEM_Legacy
  *
  * SUBTEST: basic-blt
- * Category: Infrastructure
  * Description: Examine blitter access path.
- * Feature: prime
- * Functionality: mock device
- * Run type: FULL
- * Sub-category: DRM
- * Test category: GEM_Legacy
  *
  * SUBTEST: basic-fence-blt
- * Category: Infrastructure
  * Description: Examine blitter access path fencing.
- * Feature: prime
- * Functionality: mock device
- * Run type: FULL
- * Sub-category: DRM
- * Test category: GEM_Legacy
  *
  * SUBTEST: basic-fence-flip
  * Description: Examine vgem bo front/back flip fencing.
- * Feature: prime
- * Run type: BAT
  *
  * SUBTEST: basic-fence-mmap
  * Description: Examine GTT access path fencing.
  * Feature: gtt, prime
- * Run type: BAT
  *
  * SUBTEST: basic-fence-read
  * Description: Examine read access path fencing.
  * Feature: gtt, prime
- * Run type: BAT
  *
  * SUBTEST: basic-gtt
  * Description: Examine access path through GTT.
  * Feature: gtt, prime
- * Run type: BAT
  *
  * SUBTEST: basic-read
  * Description: Examine read access path.
  * Feature: gtt, prime
- * Run type: BAT
  *
  * SUBTEST: basic-write
  * Description: Examine write access path.
  * Feature: gtt, prime
- * Run type: BAT
  *
  * SUBTEST: busy
- * Category: Infrastructure
  * Description: Examine busy check of polling for vgem fence.
- * Feature: prime
- * Functionality: mock device
- * Run type: FULL
- * Sub-category: DRM
- * Test category: GEM_Legacy
  *
  * SUBTEST: coherency-blt
- * Category: Infrastructure
  * Description: Examine blitter access path WC coherency.
- * Feature: prime
- * Functionality: mock device
- * Run type: FULL
- * Sub-category: DRM
- * Test category: GEM_Legacy
  *
  * SUBTEST: coherency-gtt
  * Description: Examine concurrent access of vgem bo.
  * Feature: gtt, prime
- * Run type: FULL
  *
  * SUBTEST: fence-flip-hang
  * Description: Examine vgem bo front/back flip fencing with a pending gpu hang.
  * Feature: blacklist, prime, synchronization
- * Run type: FULL
  *
  * SUBTEST: fence-read-hang
  * Description: Examine read access path fencing with a pending gpu hang.
  * Feature: blacklist, prime, synchronization
- * Run type: FULL
  *
  * SUBTEST: fence-wait
- * Category: Infrastructure
  * Description: Examine basic dma-buf fence interop.
- * Feature: prime, synchronization
- * Functionality: mock device
- * Run type: FULL
- * Sub-category: DRM
- * Test category: GEM_Legacy
  *
  * SUBTEST: fence-write-hang
  * Description: Examine write access path fencing with a pending gpu hang.
  * Feature: blacklist, prime, synchronization
- * Run type: FULL
  *
  * SUBTEST: shrink
- * Category: Infrastructure
  * Description: Examine link establishment between shrinker and vgem bo.
- * Feature: prime
- * Functionality: mock device
- * Run type: FULL
- * Sub-category: DRM
- * Test category: GEM_Legacy
  *
  * SUBTEST: sync
- * Category: Infrastructure
  * Description: Examine sync on vgem fence.
- * Feature: prime
- * Functionality: mock device
- * Run type: FULL
- * Sub-category: DRM
- * Test category: GEM_Legacy
  *
  * SUBTEST: wait
- * Category: Infrastructure
  * Description: Examine wait on vgem fence.
- * Feature: prime
- * Functionality: mock device
- * Run type: FULL
- * Sub-category: DRM
- * Test category: GEM_Legacy
  */
 
 IGT_TEST_DESCRIPTION("Basic check of polling for prime/vgem fences.");
@@ -351,11 +298,11 @@ static void test_fence_blt(int i915, int vgem)
 		write(master[1], &child, sizeof(child));
 		read(slave[0], &child, sizeof(child));
 
-		igt_blitter_src_copy(i915, ahnd, 0, NULL, prime, 0, scratch.pitch,
-				     I915_TILING_NONE, 0, 0, scratch.size,
-				     scratch.width, scratch.height, scratch.bpp,
-				     native, 0, scratch.pitch,
-				     I915_TILING_NONE, 0, 0, scratch.size);
+		igt_blitter_copy(i915, ahnd, 0, NULL, prime, 0, scratch.pitch,
+				 I915_TILING_NONE, 0, 0, scratch.size,
+				 scratch.width, scratch.height, scratch.bpp,
+				 native, 0, scratch.pitch,
+				 I915_TILING_NONE, 0, 0, scratch.size);
 		gem_sync(i915, native);
 
 		for (i = 0; i < scratch.height; i++)
@@ -480,11 +427,11 @@ static void test_blt(int vgem, int i915)
 		ptr[scratch.pitch * i / sizeof(*ptr)] = i;
 	munmap(ptr, scratch.size);
 
-	igt_blitter_src_copy(i915, ahnd, 0, NULL, native, 0, scratch.pitch,
-			     I915_TILING_NONE, 0, 0, scratch.size,
-			     scratch.width, scratch.height, scratch.bpp,
-			     prime, 0, scratch.pitch, I915_TILING_NONE, 0, 0,
-			     scratch.size);
+	igt_blitter_copy(i915, ahnd, 0, NULL, native, 0, scratch.pitch,
+			 I915_TILING_NONE, 0, 0, scratch.size,
+			 scratch.width, scratch.height, scratch.bpp,
+			 prime, 0, scratch.pitch, I915_TILING_NONE, 0, 0,
+			 scratch.size);
 	prime_sync_start(dmabuf, true);
 	prime_sync_end(dmabuf, true);
 	close(dmabuf);
@@ -496,11 +443,11 @@ static void test_blt(int vgem, int i915)
 	}
 	munmap(ptr, scratch.size);
 
-	igt_blitter_src_copy(i915, ahnd, 0, NULL, prime, 0, scratch.pitch,
-			     I915_TILING_NONE, 0, 0, scratch.size,
-			     scratch.width, scratch.height, scratch.bpp,
-			     native, 0, scratch.pitch, I915_TILING_NONE, 0, 0,
-			     scratch.size);
+	igt_blitter_copy(i915, ahnd, 0, NULL, prime, 0, scratch.pitch,
+			 I915_TILING_NONE, 0, 0, scratch.size,
+			 scratch.width, scratch.height, scratch.bpp,
+			 native, 0, scratch.pitch, I915_TILING_NONE, 0, 0,
+			 scratch.size);
 	gem_sync(i915, native);
 
 	ptr = gem_mmap__device_coherent(i915, native, 0, scratch.size, PROT_READ);
@@ -620,22 +567,22 @@ static void test_blt_interleaved(int vgem, int i915)
 
 	for (i = 0; i < SLOW_QUICK(scratch.height, 64); i++) {
 		local[scratch.pitch * i / sizeof(*local)] = i;
-		igt_blitter_src_copy(i915, ahnd, 0, NULL, native, 0,
-				     scratch.pitch, I915_TILING_NONE, 0, i,
-				     scratch.size, scratch.width, 1,
-				     scratch.bpp, prime, 0, scratch.pitch,
-				     I915_TILING_NONE, 0, i, scratch.size);
+		igt_blitter_copy(i915, ahnd, 0, NULL, native, 0,
+				 scratch.pitch, I915_TILING_NONE, 0, i,
+				 scratch.size, scratch.width, 1,
+				 scratch.bpp, prime, 0, scratch.pitch,
+				 I915_TILING_NONE, 0, i, scratch.size);
 		prime_sync_start(dmabuf, true);
 		igt_assert_eq_u32(foreign[scratch.pitch * i / sizeof(*foreign)],
 				  i);
 		prime_sync_end(dmabuf, true);
 
 		foreign[scratch.pitch * i / sizeof(*foreign)] = ~i;
-		igt_blitter_src_copy(i915, ahnd, 0, NULL, prime, 0, scratch.pitch,
-				     I915_TILING_NONE, 0, i, scratch.size,
-				     scratch.width, 1,
-				     scratch.bpp, native, 0, scratch.pitch,
-				     I915_TILING_NONE, 0, i, scratch.size);
+		igt_blitter_copy(i915, ahnd, 0, NULL, prime, 0, scratch.pitch,
+				 I915_TILING_NONE, 0, i, scratch.size,
+				 scratch.width, 1,
+				 scratch.bpp, native, 0, scratch.pitch,
+				 I915_TILING_NONE, 0, i, scratch.size);
 		gem_sync(i915, native);
 		igt_assert_eq_u32(local[scratch.pitch * i / sizeof(*local)],
 				  ~i);
